@@ -52,9 +52,19 @@ class CommonConsumerBase : public MetadataConsumerBase, public MarkerConsumerBas
 
     virtual void SetCurrentBlockIndex(uint64_t block_index) override { block_index_ = block_index; }
 
-    virtual void ProcessSetEnvironmentVariablesCommand(format::SetEnvironmentVariablesCommand& header,
-                                                       const char*                             env_string)
+    virtual void SetCurrentFrameNumber(uint64_t frame_number) { frame_number_ = frame_number; }
+
+    virtual void ProcessSetEnvironmentVariablesCommand(const format::SetEnvironmentVariablesCommand& header,
+                                                       const char*                                   env_string)
     {}
+
+    virtual void PushRecaptureHandleId(const format::HandleId* id) {}
+    virtual void PushRecaptureHandleIds(const format::HandleId* id_array, uint64_t id_count) {}
+    virtual void ClearRecaptureHandleIds() {}
+    virtual bool IsRecapture() { return false; }
+
+  protected:
+    uint64_t frame_number_{ 0 };
 };
 
 /* Utility */
