@@ -28,10 +28,10 @@
 #include "decode/custom_vulkan_struct_decoders.h"
 
 #include "util/platform.h"
-#include "util/defines.h"
 
 #include "nlohmann/json.hpp"
 #include "vulkan/vulkan.h"
+#include <span>
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
@@ -250,6 +250,7 @@ void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkWriteDescriptorS
                 // Nothing to do here for acceleration-structures and inline-uniform-blocks,
                 // as the rest of the data is stored in the pNext chain
                 break;
+            case VK_DESCRIPTOR_TYPE_TENSOR_ARM:
             case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV:
             case VK_DESCRIPTOR_TYPE_MUTABLE_EXT:
             case VK_DESCRIPTOR_TYPE_PARTITIONED_ACCELERATION_STRUCTURE_NV:
@@ -563,5 +564,133 @@ void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkImageToMemoryCop
     }
 }
 
+void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkLayerSettingEXT* data, const JsonOptions& options)
+{
+    if (data && data->decoded_value)
+    {
+        const VkLayerSettingEXT&         decoded_value = *data->decoded_value;
+        const Decoded_VkLayerSettingEXT& meta_struct   = *data;
+
+        FieldToJson(jdata["pLayerName"], &meta_struct.pLayerName, options);
+        FieldToJson(jdata["pSettingName"], &meta_struct.pSettingName, options);
+        FieldToJson(jdata["type"], decoded_value.type, options);
+        FieldToJson(jdata["valueCount"], decoded_value.valueCount, options);
+
+        auto typed_json_array = [](const auto* ptr, size_t num_elements) -> nlohmann::json {
+            return nlohmann::json(std::span(ptr, ptr + num_elements));
+        };
+
+        auto& value_out_json = jdata["pValues"];
+
+        switch (decoded_value.type)
+        {
+            case VK_LAYER_SETTING_TYPE_BOOL32_EXT:
+                value_out_json =
+                    typed_json_array(static_cast<const VkBool32*>(decoded_value.pValues), decoded_value.valueCount);
+                break;
+            case VK_LAYER_SETTING_TYPE_INT32_EXT:
+                value_out_json =
+                    typed_json_array(static_cast<const int32_t*>(decoded_value.pValues), decoded_value.valueCount);
+                break;
+            case VK_LAYER_SETTING_TYPE_UINT32_EXT:
+                value_out_json =
+                    typed_json_array(static_cast<const uint32_t*>(decoded_value.pValues), decoded_value.valueCount);
+                break;
+            case VK_LAYER_SETTING_TYPE_INT64_EXT:
+                value_out_json =
+                    typed_json_array(static_cast<const int64_t*>(decoded_value.pValues), decoded_value.valueCount);
+                break;
+            case VK_LAYER_SETTING_TYPE_UINT64_EXT:
+                value_out_json =
+                    typed_json_array(static_cast<const uint64_t*>(decoded_value.pValues), decoded_value.valueCount);
+                break;
+            case VK_LAYER_SETTING_TYPE_FLOAT32_EXT:
+                value_out_json =
+                    typed_json_array(static_cast<const float*>(decoded_value.pValues), decoded_value.valueCount);
+                break;
+            case VK_LAYER_SETTING_TYPE_FLOAT64_EXT:
+                value_out_json =
+                    typed_json_array(static_cast<const double*>(decoded_value.pValues), decoded_value.valueCount);
+                break;
+            case VK_LAYER_SETTING_TYPE_STRING_EXT:
+            {
+                const auto* string_array = static_cast<const char* const*>(decoded_value.pValues);
+                value_out_json           = nlohmann::json::array();
+
+                for (uint32_t i = 0; i < decoded_value.valueCount; ++i)
+                {
+                    auto str_view = std::string_view(string_array[i]);
+                    value_out_json.push_back(str_view);
+                }
+            }
+            break;
+            case VK_LAYER_SETTING_TYPE_MAX_ENUM_EXT:
+                GFXRECON_ASSERT(false);
+                break;
+        }
+    }
+}
+
+void FieldToJson(nlohmann::ordered_json&            jdata,
+                 VkDescriptorType                   discriminant,
+                 const Decoded_VkDescriptorDataEXT* data,
+                 const util::JsonOptions&           options)
+{
+    if (data && data->decoded_value)
+    {
+        switch (discriminant)
+        {
+            case VK_DESCRIPTOR_TYPE_SAMPLER:
+                HandleToJson(jdata["pSampler"], &data->pSampler, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER:
+                FieldToJson(jdata["pCombinedImageSampler"], data->pCombinedImageSampler, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT:
+                FieldToJson(jdata["pInputAttachmentImage"], data->pInputAttachmentImage, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE:
+                FieldToJson(jdata["pSampledImage"], data->pSampledImage, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE:
+                FieldToJson(jdata["pStorageImage"], data->pStorageImage, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
+                FieldToJson(jdata["pUniformTexelBuffer"], data->pUniformTexelBuffer, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
+                FieldToJson(jdata["pStorageTexelBuffer"], data->pStorageTexelBuffer, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER:
+                FieldToJson(jdata["pUniformBuffer"], data->pUniformBuffer, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER:
+                FieldToJson(jdata["pStorageBuffer"], data->pStorageBuffer, options);
+                break;
+            case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR:
+                FieldToJsonAsHex(jdata["accelerationStructure"], data->decoded_value->accelerationStructure, options);
+                break;
+            default:
+                jdata = "Invalid DescriptorType: " + std::to_string(discriminant);
+                break;
+        }
+    }
+}
+
+void FieldToJson(nlohmann::ordered_json&               jdata,
+                 const Decoded_VkDescriptorGetInfoEXT* data,
+                 const util::JsonOptions&              options)
+{
+    if (data && data->decoded_value)
+    {
+        const auto& decoded_value = *data->decoded_value;
+        const auto& meta_struct   = *data;
+        FieldToJson(jdata["sType"], decoded_value.sType, options);
+        FieldToJson(jdata["type"], decoded_value.type, options);
+        FieldToJson(jdata["data"], decoded_value.type, meta_struct.data, options);
+        FieldToJson(jdata["pNext"], meta_struct.pNext, options);
+    }
+}
+
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
