@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 2022 LunarG, Inc.
+** Copyright (c) 2022-2024 LunarG, Inc.
 ** Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
@@ -27,6 +27,13 @@
 #include "decode/dx12_experimental_resource_value_tracker.h"
 #include "decode/dx12_object_mapping_util.h"
 
+#if defined(GFXRECON_DXC_SUPPORT)
+#include <d3d12shader.h>
+#include <dxcapi.h>
+#endif
+
+#include <codecvt>
+
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
 
@@ -82,7 +89,8 @@ void GetRootSignatureResourceValueInfos(const T* root_signature_desc, std::set<R
                                      ResourceValueType::kGpuDescriptorHandle,
                                      sizeof(D3D12_GPU_DESCRIPTOR_HANDLE::ptr),
                                      nullptr,
-                                     { nullptr, nullptr, 0 } });
+                                     { nullptr, nullptr, 0 },
+                                     0 });
                 byte_offset = aligned_offset + sizeof(D3D12_GPU_DESCRIPTOR_HANDLE::ptr);
             }
             break;
@@ -95,7 +103,8 @@ void GetRootSignatureResourceValueInfos(const T* root_signature_desc, std::set<R
                                      ResourceValueType::kGpuVirtualAddress,
                                      sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
                                      nullptr,
-                                     { nullptr, nullptr, 0 } });
+                                     { nullptr, nullptr, 0 },
+                                     0 });
                 byte_offset = aligned_offset + sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
             }
             break;
@@ -121,7 +130,8 @@ void CopyResourceValuesFromDstToSrc(std::set<ResourceValueInfo>&          src,
                      (*dst_iter).type,
                      static_cast<uint64_t>((*dst_iter).size),
                      (*dst_iter).state_object,
-                     (*dst_iter).arg_buffer_extra_info });
+                     (*dst_iter).arg_buffer_extra_info,
+                     (*dst_iter).max_command_count });
     }
 }
 
@@ -141,14 +151,38 @@ void CopyMappedResourceValuesFromSrcToDst(std::map<uint64_t, T>&
     }
 }
 
+std::string DemangleDxilExportName(const std::string& mangled_name)
+{
+    size_t demangled_name_start = mangled_name.find_first_of("?");
+    size_t demangled_name_end   = mangled_name.find_first_of("@");
+
+    std::string demangled_name = "";
+    if (demangled_name_start != std::string::npos && demangled_name_end != std::string::npos)
+    {
+        // The char after '?' is the first char of the unmangled name so increment start pos.
+        ++demangled_name_start;
+        demangled_name = mangled_name.substr(demangled_name_start, demangled_name_end - demangled_name_start);
+    }
+
+    if (!demangled_name.empty())
+    {
+        GFXRECON_LOG_DEBUG("Found demangled DXIL export name '%s'.", demangled_name.c_str());
+    }
+    else
+    {
+        GFXRECON_LOG_WARNING("Failed to demangle DXIL export name '%s'.", mangled_name.c_str());
+    }
+    return demangled_name;
+}
+
 } // namespace
 
 Dx12ResourceValueMapper::Dx12ResourceValueMapper(std::function<DxObjectInfo*(format::HandleId id)> get_object_info_func,
                                                  const graphics::Dx12ShaderIdMap&                  shader_id_map,
                                                  const graphics::Dx12GpuVaMap&                     gpu_va_map,
                                                  const decode::Dx12DescriptorMap&                  descriptor_map) :
-    get_object_info_func_(get_object_info_func),
-    shader_id_map_(shader_id_map), gpu_va_map_(gpu_va_map), descriptor_map_(descriptor_map), do_value_mapping_(true)
+    get_object_info_func_(get_object_info_func), shader_id_map_(shader_id_map), gpu_va_map_(gpu_va_map),
+    descriptor_map_(descriptor_map), do_value_mapping_(true)
 {}
 
 void Dx12ResourceValueMapper::EnableResourceValueTracker(std::function<uint64_t(void)> get_current_block_index_func,
@@ -377,7 +411,8 @@ void Dx12ResourceValueMapper::PostProcessCreateCommandSignature(HandlePointerDec
                                               ResourceValueType::kGpuVirtualAddress,
                                               sizeof(D3D12_VERTEX_BUFFER_VIEW::BufferLocation),
                                               nullptr,
-                                              { nullptr, nullptr, 0 } });
+                                              { nullptr, nullptr, 0 },
+                                              0 });
                 byte_offset += sizeof(D3D12_VERTEX_BUFFER_VIEW);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_INDEX_BUFFER_VIEW:
@@ -385,7 +420,8 @@ void Dx12ResourceValueMapper::PostProcessCreateCommandSignature(HandlePointerDec
                                               ResourceValueType::kGpuVirtualAddress,
                                               sizeof(D3D12_INDEX_BUFFER_VIEW::BufferLocation),
                                               nullptr,
-                                              { nullptr, nullptr, 0 } });
+                                              { nullptr, nullptr, 0 },
+                                              0 });
                 byte_offset += sizeof(D3D12_INDEX_BUFFER_VIEW);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_CONSTANT:
@@ -398,7 +434,8 @@ void Dx12ResourceValueMapper::PostProcessCreateCommandSignature(HandlePointerDec
                                               ResourceValueType::kGpuVirtualAddress,
                                               sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
                                               nullptr,
-                                              { nullptr, nullptr, 0 } });
+                                              { nullptr, nullptr, 0 },
+                                              0 });
                 byte_offset += sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_RAYS:
@@ -406,7 +443,8 @@ void Dx12ResourceValueMapper::PostProcessCreateCommandSignature(HandlePointerDec
                                               ResourceValueType::kIndirectArgumentDispatchRays,
                                               sizeof(D3D12_DISPATCH_RAYS_DESC),
                                               nullptr,
-                                              { nullptr, nullptr, 0 } });
+                                              { nullptr, nullptr, 0 },
+                                              0 });
                 byte_offset += sizeof(D3D12_DISPATCH_RAYS_DESC);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_MESH:
@@ -449,7 +487,8 @@ void Dx12ResourceValueMapper::PostProcessExecuteIndirect(DxObjectInfo* command_l
                   ResourceValueType::kExecuteIndirectCountBuffer,
                   sizeof(uint32_t),
                   state_object_extra_info,
-                  { command_signature_extra_info, argument_buffer_object_info, argument_buffer_offset } });
+                  { command_signature_extra_info, argument_buffer_object_info, argument_buffer_offset },
+                  max_command_count });
         }
         else
         {
@@ -462,6 +501,7 @@ void Dx12ResourceValueMapper::PostProcessExecuteIndirect(DxObjectInfo* command_l
                 command_signature_extra_info->byte_stride,
                 state_object_extra_info);
         }
+        SourceCopyResources(command_list_extra_info->resource_copies, command_list_extra_info->resource_value_info_map);
     }
 }
 
@@ -477,11 +517,18 @@ void Dx12ResourceValueMapper::PostProcessBuildRaytracingAccelerationStructure(
 
         format::HandleId resource_id = format::kNullHandleId;
         bool             found       = false;
-        reverse_gpu_va_map_.Map(build_desc->Inputs.InstanceDescs,
-                                &resource_id,
-                                &found,
-                                build_desc->Inputs.InstanceDescs +
-                                    build_desc->Inputs.NumDescs * sizeof(D3D12_RAYTRACING_INSTANCE_DESC));
+
+        auto min_end_gpu_va = build_desc->Inputs.InstanceDescs;
+        if (build_desc->Inputs.DescsLayout == D3D12_ELEMENTS_LAYOUT_ARRAY)
+        {
+            min_end_gpu_va += build_desc->Inputs.NumDescs * sizeof(D3D12_RAYTRACING_INSTANCE_DESC);
+        }
+        else
+        {
+            min_end_gpu_va += build_desc->Inputs.NumDescs * sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
+        }
+
+        reverse_gpu_va_map_.Map(build_desc->Inputs.InstanceDescs, &resource_id, &found, min_end_gpu_va);
 
         if (resource_id != format::kNullHandleId)
         {
@@ -511,15 +558,30 @@ void Dx12ResourceValueMapper::PostProcessBuildRaytracingAccelerationStructure(
                           ResourceValueType::kGpuVirtualAddress,
                           sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
                           nullptr,
-                          { nullptr, nullptr, 0 } });
+                          { nullptr, nullptr, 0 },
+                          0 });
+                }
+            }
+            else if (build_desc->Inputs.DescsLayout == D3D12_ELEMENTS_LAYOUT_ARRAY_OF_POINTERS)
+            {
+                constexpr auto instance_desc_pointer_stride = sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
+                for (UINT i = 0; i < build_desc->Inputs.NumDescs; ++i)
+                {
+                    resource_value_infos.insert({ offset_to_instance_descs_start + instance_desc_pointer_stride * i,
+                                                  ResourceValueType::kRaytracingInstanceDescPointer,
+                                                  sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
+                                                  nullptr,
+                                                  { nullptr, nullptr, 0 },
+                                                  0 });
                 }
             }
             else
             {
-                // TODO: Support D3D12_ELEMENTS_LAYOUT_ARRAY_OF_POINTERS.
-                GFXRECON_LOG_WARNING("Application built acceleration structure with unsupported layout: "
-                                     "D3D12_ELEMENTS_LAYOUT_ARRAY_OF_POINTERS");
+                GFXRECON_LOG_ERROR("Unknown BuildRaytracingAccelerationStructure DescsLayout: %d",
+                                   static_cast<int>(build_desc->Inputs.DescsLayout));
             }
+            SourceCopyResources(command_list_extra_info->resource_copies,
+                                command_list_extra_info->resource_value_info_map);
         }
         else
         {
@@ -734,6 +796,7 @@ void Dx12ResourceValueMapper::PostProcessDispatchRays(
         state_object_extra_info = GetExtraInfo<D3D12StateObjectInfo>(command_list_extra_info->active_state_object);
     }
     GetDispatchRaysResourceValues(command_list_extra_info->resource_value_info_map, state_object_extra_info, *desc);
+    SourceCopyResources(command_list_extra_info->resource_copies, command_list_extra_info->resource_value_info_map);
 }
 
 void Dx12ResourceValueMapper::PostProcessSetPipelineState1(DxObjectInfo* command_list4_object_info,
@@ -853,6 +916,17 @@ void Dx12ResourceValueMapper::RemoveGpuDescriptorHeap(uint64_t capture_gpu_start
     }
 }
 
+void Dx12ResourceValueMapper::SourceCopyResources(std::vector<ResourceCopyInfo>& copy_infos,
+                                                  ResourceValueInfoMap&          resource_value_info_map)
+{
+    const auto copies_size = copy_infos.size();
+    for (size_t i = 0; i < copies_size; ++i)
+    {
+        auto& copy_info = copy_infos[copies_size - i - 1];
+        CopyResourceValues(copy_info, resource_value_info_map);
+    }
+}
+
 void Dx12ResourceValueMapper::CopyResourceValues(const ResourceCopyInfo& copy_info,
                                                  ResourceValueInfoMap&   resource_value_info_map)
 {
@@ -860,7 +934,10 @@ void Dx12ResourceValueMapper::CopyResourceValues(const ResourceCopyInfo& copy_in
     auto dst_iter = resource_value_info_map.find(copy_info.dst_resource_object_info);
     if (dst_iter != resource_value_info_map.end())
     {
-        GFXRECON_ASSERT(!dst_iter->second.empty());
+        if (dst_iter->second.empty())
+        {
+            return;
+        }
 
         auto& dst_values = dst_iter->second;
         auto& src_values = resource_value_info_map[copy_info.src_resource_object_info];
@@ -869,12 +946,13 @@ void Dx12ResourceValueMapper::CopyResourceValues(const ResourceCopyInfo& copy_in
         if (copy_info.num_bytes != 0)
         {
             auto dst_values_begin = dst_values.lower_bound(
-                { copy_info.dst_offset, ResourceValueType::kUnknown, 0, nullptr, { nullptr, nullptr, 0 } });
+                { copy_info.dst_offset, ResourceValueType::kUnknown, 0, nullptr, { nullptr, nullptr, 0 }, 0 });
             auto dst_values_end = dst_values.upper_bound({ copy_info.dst_offset + copy_info.num_bytes,
                                                            ResourceValueType::kUnknown,
                                                            0,
                                                            nullptr,
-                                                           { nullptr, nullptr, 0 } });
+                                                           { nullptr, nullptr, 0 },
+                                                           0 });
             CopyResourceValuesFromDstToSrc(src_values, dst_values, dst_values_begin, dst_values_end, copy_info);
             dst_values.erase(dst_values_begin, dst_values_end);
         }
@@ -983,16 +1061,11 @@ void Dx12ResourceValueMapper::ProcessResourceMappings(ProcessResourceMappingsArg
 
     auto resource_value_info_map = std::move(args.resource_value_info_map);
     args.resource_value_info_map.clear();
-    const auto                                        copies_size = args.resource_copies.size();
     std::map<DxObjectInfo*, MappedResourceRevertInfo> resource_data_to_revert;
     while (!resource_value_info_map.empty())
     {
         // Process resource copies so values are mapped in the source resource.
-        for (size_t i = 0; i < copies_size; ++i)
-        {
-            auto& copy_info = args.resource_copies[copies_size - i - 1];
-            CopyResourceValues(copy_info, resource_value_info_map);
-        }
+        SourceCopyResources(args.resource_copies, resource_value_info_map);
 
         // Apply the resource value mappings to the resources on the GPU.
         ResourceValueInfoMap indirect_values_map;
@@ -1003,6 +1076,7 @@ void Dx12ResourceValueMapper::ProcessResourceMappings(ProcessResourceMappingsArg
     }
 
     // Track mapped values that were copied to other resources.
+    const auto copies_size = args.resource_copies.size();
     for (size_t i = 0; i < copies_size; ++i)
     {
         auto& copy_info = args.resource_copies[i];
@@ -1235,7 +1309,7 @@ bool Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
         uint8_t* desc_data_ptr = result_data.data() + final_offset;
 
         // First map the StartAddress GPU VAs from D3D12_DISPATCH_RAYS_DESC in resource data.
-        ResourceValueInfo rvi{ 0, ResourceValueType::kUnknown, 0, nullptr, { nullptr, nullptr, 0 } };
+        ResourceValueInfo rvi{ 0, ResourceValueType::kUnknown, 0, nullptr, { nullptr, nullptr, 0 }, 0 };
         rvi.size         = sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
         rvi.type         = ResourceValueType::kGpuVirtualAddress;
         rvi.state_object = value_info.state_object;
@@ -1280,6 +1354,10 @@ bool Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
         // Insert new ArgumentBuffer RV infos, which will queue them for translation.
         if (command_count != 0)
         {
+            if (value_info.max_command_count > 0)
+            {
+                command_count = std::min(value_info.max_command_count, command_count);
+            }
             GetExecuteIndirectResourceValues(
                 indirect_values_map[value_info.arg_buffer_extra_info.argument_buffer],
                 value_info.arg_buffer_extra_info.command_signature_info->resource_value_infos,
@@ -1290,6 +1368,73 @@ bool Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
         }
         return false;
     }
+    else if (value_info.type == ResourceValueType::kRaytracingInstanceDescPointer)
+    {
+        GFXRECON_ASSERT(value_info.size == sizeof(D3D12_GPU_VIRTUAL_ADDRESS));
+
+        // Map the GPU_VA in the array of instance desc pointers.
+        ResourceValueInfo rvi = value_info;
+        rvi.type              = ResourceValueType::kGpuVirtualAddress;
+        MapValue(rvi, result_data, resource_id, resource_info, indirect_values_map);
+
+        // Read instance desc GPU_VA from the array of pointers.
+        D3D12_GPU_VIRTUAL_ADDRESS instance_desc_gpu_va = 0;
+        util::platform::MemoryCopy(&instance_desc_gpu_va,
+                                   sizeof(instance_desc_gpu_va),
+                                   result_data.data() + value_info.offset,
+                                   sizeof(instance_desc_gpu_va));
+
+        GFXRECON_ASSERT(value_info.offset == final_offset);
+
+        // Insert new RV infos for instance desc's AccelerationStructure, which will queue it for mapping.
+        if (instance_desc_gpu_va != 0)
+        {
+            // The spec requires that instance descs are aligned to D3D12_RAYTRACING_INSTANCE_DESCS_BYTE_ALIGNMENT. If
+            // the instance descs are not aligned behavior may be undefined.
+            GFXRECON_ASSERT((instance_desc_gpu_va % D3D12_RAYTRACING_INSTANCE_DESCS_BYTE_ALIGNMENT) == 0);
+
+            // Find the resource that contains the address referenced by instance_desc_gpu_va.
+            format::HandleId instance_desc_resource_id = format::kNullHandleId;
+            bool             found                     = false;
+            reverse_gpu_va_map_.Map(instance_desc_gpu_va,
+                                    &instance_desc_resource_id,
+                                    &found,
+                                    instance_desc_gpu_va + sizeof(D3D12_RAYTRACING_INSTANCE_DESC));
+
+            if (instance_desc_resource_id != format::kNullHandleId)
+            {
+                GFXRECON_ASSERT(found);
+
+                auto resource_object_info = get_object_info_func_(instance_desc_resource_id);
+                GFXRECON_ASSERT(resource_object_info != nullptr);
+                GFXRECON_ASSERT(resource_object_info->object != nullptr);
+
+                auto instance_desc_resource = static_cast<ID3D12Resource*>(resource_object_info->object);
+                GFXRECON_ASSERT(instance_desc_gpu_va >= instance_desc_resource->GetGPUVirtualAddress());
+                auto offset_to_instance_desc_start =
+                    instance_desc_gpu_va - instance_desc_resource->GetGPUVirtualAddress();
+
+                constexpr auto accel_struct_gpu_va_offset =
+                    offsetof(D3D12_RAYTRACING_INSTANCE_DESC, AccelerationStructure);
+
+                auto& resource_value_infos = indirect_values_map[resource_object_info];
+                resource_value_infos.insert({ offset_to_instance_desc_start + accel_struct_gpu_va_offset,
+                                              ResourceValueType::kGpuVirtualAddress,
+                                              sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
+                                              nullptr,
+                                              { nullptr, nullptr, 0 },
+                                              0 });
+            }
+            else
+            {
+                GFXRECON_LOG_ERROR("Failed to find the resource containing the D3D12_GPU_VIRTUAL_ADDRESS (%" PRIu64
+                                   ") of InstanceDescs in call to BuildRaytracingAccelerationStructure. GPU addresses "
+                                   "pointed to by InstanceDescs may be incorrect.",
+                                   instance_desc_gpu_va);
+            }
+        }
+        return true;
+    }
     else
     {
         GFXRECON_ASSERT(false && "Unrecognized resource value type.");
@@ -1482,7 +1627,8 @@ void Dx12ResourceValueMapper::GetShaderTableResourceValues(ResourceValueInfoMap&
                                       ResourceValueType::kShaderIdentifier,
                                       shader_record_size,
                                       state_object_extra_info,
-                                      { nullptr, nullptr, 0 } });
+                                      { nullptr, nullptr, 0 },
+                                      0 });
         byte_offset += shader_record_size;
     }
 }
@@ -1537,7 +1683,8 @@ void Dx12ResourceValueMapper::GetExecuteIndirectResourceValues(
                                                  resource_value_info.type,
                                                  resource_value_info.size,
                                                  state_object,
-                                                 resource_value_info.arg_buffer_extra_info });
+                                                 resource_value_info.arg_buffer_extra_info,
+                                                 0 });
         }
         command_offset += stride;
     }
@@ -1606,7 +1753,8 @@ void Dx12ResourceValueMapper::GetStateObjectLrsAssociationInfo(
         }
         else if (subobject_type == D3D12_STATE_SUBOBJECT_TYPE_DXIL_LIBRARY)
         {
-            // TODO: Parse local root signatures and their shader associations from the DXIL library.
+            // TODO: DXIL libraries can also contain local root signature definitions as well as subobject associations.
+            // That information should be parsed here as well to ensure correct LRS handling.
             GFXRECON_LOG_DEBUG_ONCE("A state object is being created with a DXIL library subobject. Some usages of "
                                     "DXIL library subobjects may not be fully supported by GFXR replay.");
 
@@ -1615,10 +1763,90 @@ void Dx12ResourceValueMapper::GetStateObjectLrsAssociationInfo(
             auto num_exports           = dxil_lib_desc_decoder->GetPointer()->NumExports;
             if (num_exports == 0)
             {
-                // TODO: Parse the names of all shaders exported from the DXIL library.
+#if defined(GFXRECON_DXC_SUPPORT)
+                // If D3D12_DXIL_LIBRARY_DESC::NumExports == 0, everything in the DXIL library is exported. Use
+                // reflection to get the list of exported shader names.
+                HRESULT                         hr;
+                graphics::dx12::IDxcUtilsComPtr dxc_utils = nullptr;
+                hr                                        = DxcCreateInstance(CLSID_DxcUtils, IID_PPV_ARGS(&dxc_utils));
+                if (SUCCEEDED(hr))
+                {
+                    graphics::dx12::IDxcContainerReflectionComPtr dxc_container_reflection = nullptr;
+                    DxcCreateInstance(CLSID_DxcContainerReflection, IID_PPV_ARGS(&dxc_container_reflection));
+                    if (SUCCEEDED(hr))
+                    {
+                        // Create a DXC blob from the DXIL library's bytes.
+                        graphics::dx12::IDxcBlobEncodingComPtr dxc_blob_encoding;
+                        hr = dxc_utils->CreateBlobFromPinned(
+                            dxil_lib_desc_decoder->GetPointer()->DXILLibrary.pShaderBytecode,
+                            dxil_lib_desc_decoder->GetPointer()->DXILLibrary.BytecodeLength,
+                            DXC_CP_ACP,
+                            &dxc_blob_encoding);
+                        if (SUCCEEDED(hr))
+                        {
+                            // Load the DXIL library blob into the container reflection object.
+                            hr = dxc_container_reflection->Load(dxc_blob_encoding);
+                            if (SUCCEEDED(hr))
+                            {
+                                // Get the DXIL library reflection object.
+                                UINT32 dxil_part;
+                                hr = dxc_container_reflection->FindFirstPartKind(DXC_PART_DXIL, &dxil_part);
+                                if (SUCCEEDED(hr))
+                                {
+                                    graphics::dx12::ID3D12LibraryReflectionComPtr library_reflection;
+                                    hr = dxc_container_reflection->GetPartReflection(dxil_part,
+                                                                                     IID_PPV_ARGS(&library_reflection));
+                                    if (SUCCEEDED(hr))
+                                    {
+                                        // Parse all exported function/shader names from the DXIL library.
+                                        D3D12_LIBRARY_DESC library_desc;
+                                        hr = library_reflection->GetDesc(&library_desc);
+                                        if (SUCCEEDED(hr))
+                                        {
+                                            for (UINT i = 0; i < library_desc.FunctionCount; i++)
+                                            {
+                                                // The pointer returned by GetFunctionByIndex is owned by the
+                                                // ID3D12LibraryReflection object and does not need to be memory managed
+                                                // in this scope.
+                                                ID3D12FunctionReflection* function_reflection =
+                                                    library_reflection->GetFunctionByIndex(i);
+
+                                                D3D12_FUNCTION_DESC function_desc;
+                                                hr = function_reflection->GetDesc(&function_desc);
+                                                if (SUCCEEDED(hr))
+                                                {
+                                                    // Export names are mangled so parse the unmangled name.
+                                                    auto function_name = DemangleDxilExportName(function_desc.Name);
+                                                    if (!function_name.empty())
+                                                    {
+                                                        std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>>
+                                                            converter;
+                                                        export_names.insert(converter.from_bytes(function_name));
+                                                    }
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+                if (FAILED(hr))
+                {
+                    GFXRECON_LOG_WARNING("Failed to parse shader exports from the DXIL subobject. Shader ID to LRS "
+                                         "associations may be incorrect.");
+                }
+#else  // GFXRECON_DXC_SUPPORT
+                GFXRECON_LOG_WARNING_ONCE(
+                    "GFXReconstruct was built without DirectX Shader Compiler support and cannot parse the exports "
+                    "from DXIL_LIBRARY subobjects. This may lead to incorrect DXR behavior. To fix this, be sure that "
+                    "the DXC depedency is successfully found during CMake project configuration.");
+#endif // GFXRECON_DXC_SUPPORT
             }
             else
             {
+                // Get the shader names specified explicitly in the D3D12_DXIL_LIBRARY_DESC.
                 for (UINT j = 0; j < num_exports; ++j)
                 {
                     export_names.insert(dxil_lib_desc_decoder->GetMetaStructPointer()
