@@ -1,6 +1,7 @@
 /*
 ** Copyright (c) 2023 Valve Corporation
 ** Copyright (c) 2022-2023 LunarG, Inc.
+** Copyright (c) 2025 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -24,9 +25,10 @@
 #ifndef GFXRECON_DECODE_METADATA_JSON_CONSUMER_H
 #define GFXRECON_DECODE_METADATA_JSON_CONSUMER_H
 
+#include "decode/custom_vulkan_struct_to_json.h"
+#include "format/format_json.h"
 #include "util/defines.h"
 #include "util/file_path.h"
-#include "format/format_json.h"
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
@@ -79,7 +81,7 @@ class MetadataJsonConsumer : public Base
         WriteBlockEnd();
     }
 
-    virtual void Process_ExeFileInfo(gfxrecon::util::filepath::FileInfo& info) override
+    virtual void Process_ExeFileInfo(const gfxrecon::util::filepath::FileInfo& info) override
     {
         const util::JsonOptions& json_options = GetOptions();
         auto&                    jdata        = WriteMetaCommandStart("ExeFileInfo");
@@ -121,7 +123,8 @@ class MetadataJsonConsumer : public Base
     }
 
     virtual void
-    ProcessCreateHardwareBufferCommand(format::HandleId                                    memory_id,
+    ProcessCreateHardwareBufferCommand(format::HandleId                                    device_id,
+                                       format::HandleId                                    memory_id,
                                        uint64_t                                            buffer_id,
                                        uint32_t                                            format,
                                        uint32_t                                            width,
@@ -133,6 +136,7 @@ class MetadataJsonConsumer : public Base
     {
         const util::JsonOptions& json_options = GetOptions();
         auto&                    jdata        = WriteMetaCommandStart("CreateHardwareBufferCommand");
+        HandleToJson(jdata["device_id"], device_id, json_options);
         HandleToJson(jdata["memory_id"], memory_id, json_options);
         HandleToJson(jdata["buffer_id"], buffer_id, json_options);
         FieldToJson(jdata["format"], format, json_options);
@@ -174,17 +178,6 @@ class MetadataJsonConsumer : public Base
         WriteBlockEnd();
     }
 
-    virtual void
-    ProcessSetDeviceMemoryPropertiesCommand(format::HandleId                             physical_device_id,
-                                            const std::vector<format::DeviceMemoryType>& memory_types,
-                                            const std::vector<format::DeviceMemoryHeap>& memory_heaps) override
-    {
-        const util::JsonOptions& json_options = GetJsonOptions();
-        auto&                    jdata        = WriteMetaCommandStart("SetDeviceMemoryPropertiesCommand");
-        HandleToJson(jdata["physical_device_id"], physical_device_id, json_options);
-        WriteBlockEnd();
-    }
-
     virtual void
     ProcessSetOpaqueAddressCommand(format::HandleId device_id, format::HandleId object_id, uint64_t address) override
     {
@@ -196,6 +189,20 @@ class MetadataJsonConsumer : public Base
         WriteBlockEnd();
     }
 
+    void ProcessSetOpaqueDescriptorDataCommand(format::HandleId device_id,
+                                               format::HandleId object_id,
+                                               uint32_t         data_size,
+                                               const uint8_t*   data) override
+    {
+        const JsonOptions& json_options = GetJsonOptions();
+        auto&              jdata        = WriteMetaCommandStart("SetOpaqueDescriptorDataCommand");
+        HandleToJson(jdata["device_id"], device_id, json_options);
+        HandleToJson(jdata["object_id"], object_id, json_options);
+        FieldToJson(jdata["data_size"], data_size, json_options);
+        RepresentBinaryFile(*(this->writer_), jdata[format::kNameData], "opaque_descriptor_data.bin", data_size, data);
+        WriteBlockEnd();
+    }
+
     virtual void ProcessSetRayTracingShaderGroupHandlesCommand(format::HandleId device_id,
                                                                format::HandleId pipeline_id,
                                                                size_t           data_size,
@@ -226,6 +233,16 @@ class MetadataJsonConsumer : public Base
         WriteBlockEnd();
     }
 
+    virtual void ProcessInitializeMetaCommand(const format::InitializeMetaCommand& command_header,
+                                              const uint8_t*                       parameters_data) override
+    {
+        const util::JsonOptions& json_options = GetJsonOptions();
+        auto&                    jdata        = WriteMetaCommandStart("InitializeMetaCommand");
+        HandleToJson(jdata["MetaCommand_id"], command_header.capture_id, json_options);
+        FieldToJson(jdata["InitializationParametersDataSizeInBytes"], command_header.data_size, json_options);
+        WriteBlockEnd();
+    }
+
     virtual void ProcessBeginResourceInitCommand(format::HandleId device_id,
                                                  uint64_t         max_resource_size,
                                                  uint64_t         max_copy_size) override
@@ -233,6 +250,8 @@ class MetadataJsonConsumer : public Base
         const JsonOptions& json_options = GetJsonOptions();
         auto&              jdata        = WriteMetaCommandStart("BeginResourceInitCommand");
         HandleToJson(jdata["device_id"], device_id, json_options);
+
+        // TODO: should be "total_copy_size"
         FieldToJson(jdata["max_resource_size"], max_resource_size, json_options);
         FieldToJson(jdata["max_copy_size"], max_copy_size, json_options);
         WriteBlockEnd();
@@ -279,6 +298,71 @@ class MetadataJsonConsumer : public Base
         WriteBlockEnd();
     }
 
+    virtual void ProcessSetEnvironmentVariablesCommand(const format::SetEnvironmentVariablesCommand& header,
+                                                       const char* env_string) override
+    {
+        const JsonOptions& json_options = GetJsonOptions();
+        auto&              json_data    = WriteMetaCommandStart("SetEnvironmentVariablesCommand");
+
+        std::vector<std::string> env_vars =
+            util::strings::SplitString(std::string_view(env_string), format::kEnvironmentStringDelimeter);
+        for (std::string& e : env_vars)
+        {
+            std::vector<std::string> var_plus_val = util::strings::SplitString(e, '=');
+            if (var_plus_val.size() == 2)
+            {
+                const char* var = var_plus_val[0].c_str();
+                const char* val = var_plus_val[1].c_str();
+                json_data[var]  = val;
+            }
+        }
+        WriteBlockEnd();
+    }
+
+    virtual void ProcessExecuteBlocksFromFile(uint32_t n_blocks, int64_t offset, const std::string& filename) override
+    {
+        const JsonOptions& json_options = GetJsonOptions();
+        auto&              jdata        = WriteMetaCommandStart("ExecuteBlocksFromFile");
+        FieldToJson(jdata["n_blocks"], n_blocks, json_options);
+        FieldToJson(jdata["offset"], offset, json_options);
+        FieldToJson(jdata["filename"], filename, json_options);
+        WriteBlockEnd();
+    }
+
+    virtual void ProcessVulkanBuildAccelerationStructuresCommand(
+        format::HandleId                                                           device_id,
+        uint32_t                                                                   info_count,
+        StructPointerDecoder<Decoded_VkAccelerationStructureBuildGeometryInfoKHR>* geometry_infos,
+        StructPointerDecoder<Decoded_VkAccelerationStructureBuildRangeInfoKHR*>*   range_infos) override
+    {
+        auto&              jdata        = WriteMetaCommandStart("ProcessVulkanBuildAccelerationStructuresCommand");
+        const JsonOptions& json_options = GetJsonOptions();
+        FieldToJson(jdata["infoCount"], info_count, json_options);
+        FieldToJson(jdata["pInfos"], geometry_infos, json_options);
+        FieldToJson(jdata["ppBuildRangeInfos"], range_infos, json_options);
+        WriteBlockEnd();
+    }
+
+    void ProcessVulkanCopyAccelerationStructuresCommand(
+        format::HandleId                                                  device_id,
+        StructPointerDecoder<Decoded_VkCopyAccelerationStructureInfoKHR>* copy_infos) override
+    {
+        auto&              jdata        = WriteMetaCommandStart("VulkanCopyAccelerationStructuresCommand");
+        const JsonOptions& json_options = GetJsonOptions();
+        FieldToJson(jdata["pInfo"], copy_infos, json_options);
+        WriteBlockEnd();
+    }
+
+    virtual void ProcessVulkanWriteAccelerationStructuresPropertiesCommand(
+        format::HandleId device_id, VkQueryType query_type, format::HandleId acceleration_structure_id) override
+    {
+        auto&              jdata        = WriteMetaCommandStart("VulkanWriteAccelerationStructuresPropertiesCommand");
+        const JsonOptions& json_options = GetJsonOptions();
+        HandleToJson(jdata["device_id"], device_id, json_options);
+        FieldToJson(jdata["queryType"], query_type, json_options);
+        HandleToJson(jdata["acceleration_structure_id"], acceleration_structure_id, json_options);
+    }
+
     /// @}
 };
 
