@@ -145,6 +145,12 @@ class VulkanDecoderBase : public ApiDecoder
                                                  format::HandleId object_id,
                                                  uint64_t         address) override;
 
+    void DispatchSetOpaqueDescriptorDataCommand(format::ThreadId thread_id,
+                                                format::HandleId device_id,
+                                                format::HandleId object_id,
+                                                uint32_t         data_size,
+                                                const uint8_t*   data) override;
+
     virtual void DispatchSetRayTracingShaderGroupHandlesCommand(format::ThreadId thread_id,
                                                                 format::HandleId device_id,
                                                                 format::HandleId pipeline_id,
@@ -187,20 +193,22 @@ class VulkanDecoderBase : public ApiDecoder
                                                         const std::vector<format::HandleId>& blases) override;
 
     virtual void DispatchInitDx12AccelerationStructureCommand(
-        const format::InitDx12AccelerationStructureCommandHeader&       command_header,
-        std::vector<format::InitDx12AccelerationStructureGeometryDesc>& geometry_descs,
-        const uint8_t*                                                  build_inputs_data) override
+        const format::InitDx12AccelerationStructureCommandHeader&             command_header,
+        const std::vector<format::InitDx12AccelerationStructureGeometryDesc>& geometry_descs,
+        const uint8_t*                                                        build_inputs_data) override
     {}
 
-    virtual void DispatchDriverInfo(format::ThreadId thread_id, format::DriverInfoBlock& info) override {}
+    virtual void DispatchDriverInfo(format::ThreadId thread_id, const format::DriverInfoBlock& info) override {}
 
-    virtual void DispatchExeFileInfo(format::ThreadId thread_id, format::ExeFileInfoBlock& info) override;
+    virtual void DispatchExeFileInfo(format::ThreadId thread_id, const format::ExeFileInfoBlock& info) override;
 
-    virtual void DispatchSetEnvironmentVariablesCommand(format::SetEnvironmentVariablesCommand& header,
-                                                        const char*                             env_string) override;
+    virtual void DispatchSetEnvironmentVariablesCommand(const format::SetEnvironmentVariablesCommand& header,
+                                                        const char* env_string) override;
 
     virtual void SetCurrentBlockIndex(uint64_t block_index) override;
 
+    virtual void SetCurrentFrameNumber(uint64_t frame_number) override;
+
     void DispatchVulkanAccelerationStructuresBuildMetaCommand(const uint8_t* parameter_buffer,
                                                               size_t         buffer_size) override;
 
