@@ -29,6 +29,7 @@ GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
 
 const std::string DUMP_ARG_BEGIN_COMMAND_BUFFER = "BeginCommandBuffer";
+const std::string DUMP_ARG_TRANSFER             = "Transfer";
 const std::string DUMP_ARG_DRAW                 = "Draw";
 const std::string DUMP_ARG_RENDER_PASS          = "RenderPass";
 const std::string DUMP_ARG_BEGIN_RENDER_PASS    = "BeginRenderPass";
@@ -37,6 +38,7 @@ const std::string DUMP_ARG_END_RENDER_PASS      = "EndRenderPass";
 const std::string DUMP_ARG_DISPATCH             = "Dispatch";
 const std::string DUMP_ARG_TRACE_RAYS           = "TraceRays";
 const std::string DUMP_ARG_QUEUE_SUBMIT         = "QueueSubmit";
+const std::string DUMP_ARG_EXECUTE_COMMANDS     = "ExecuteCommands";
 
 enum class VkDumpDrawCallType
 {
@@ -723,4 +725,4 @@ class VulkanPreProcessConsumer : public VulkanConsumer
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
 
-#endif // GFXRECON_DECODE_VULKAN_PRE_PROCESS_CONSUMER_H
\ No newline at end of file
+#endif // GFXRECON_DECODE_VULKAN_PRE_PROCESS_CONSUMER_H
