@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 2019-2020 Valve Corporation
-** Copyright (c) 2019-2023 LunarG, Inc.
+** Copyright (c) 2019-2025 LunarG, Inc.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -27,12 +27,16 @@
 #include "decode/replay_options.h"
 
 #include "decode/vulkan_resource_allocator.h"
+#include "format/format.h"
 #include "util/defines.h"
 
+#include <cstdint>
 #include <functional>
 #include <string>
-#include <unordered_set>
+#include <map>
+#include <unordered_map>
 #include <vector>
+#include <vulkan/vulkan_core.h>
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
@@ -50,6 +54,47 @@ enum class SkipGetFenceStatus
     COUNT
 };
 
+using Index             = uint64_t;
+using CommandIndices    = std::vector<Index>;
+using RenderPassIndices = std::vector<std::vector<Index>>;
+using ExecuteCommands   = std::unordered_map<Index, CommandIndices>;
+
+struct DescriptorLocation
+{
+    bool const operator==(const DescriptorLocation& other) const
+    {
+        return set == other.set && binding == other.binding && array_index == other.array_index;
+    }
+
+    bool const operator<(const DescriptorLocation& other) const
+    {
+        if (set == other.set)
+        {
+            if (binding == other.binding)
+            {
+                return array_index < other.array_index;
+            }
+            else
+            {
+                return binding < other.binding;
+            }
+        }
+        else
+        {
+            return set < other.set;
+        }
+    }
+
+    uint32_t set;
+    uint32_t binding;
+    uint32_t array_index;
+};
+
+using CommandImageSubresource =
+    std::unordered_map<decode::Index, std::map<DescriptorLocation, VkImageSubresourceRange>>;
+using CommandImageSubresourceIterator = CommandImageSubresource::const_iterator;
+using BeginCmdBufQueueSubmitPair      = std::pair<decode::Index, decode::Index>;
+
 // Default color attachment index selection for dump resources feature.
 // This default value essentially defines to dump all attachments.
 static constexpr int kUnspecifiedColorAttachment = -1;
@@ -57,10 +102,12 @@ static constexpr int kUnspecifiedColorAttachment = -1;
 struct VulkanReplayOptions : public ReplayOptions
 {
     bool                         enable_vulkan{ true };
+    bool                         capture{ false };
     bool                         omit_pipeline_cache_data{ false };
     bool                         use_colorspace_fallback{ false };
     bool                         offscreen_swapchain_frame_boundary{ false };
     util::SwapchainOption        swapchain_option{ util::SwapchainOption::kVirtual };
+    util::PresentModeOption      present_mode_option{ util::PresentModeOption::kCapture };
     bool                         virtual_swapchain_skip_blit{ false };
     int32_t                      override_gpu_group_index{ -1 };
     int32_t                      surface_index{ -1 };
@@ -71,17 +118,30 @@ struct VulkanReplayOptions : public ReplayOptions
     SkipGetFenceStatus           skip_get_fence_status{ SkipGetFenceStatus::NoSkip };
     std::vector<util::UintRange> skip_get_fence_ranges;
     bool                         wait_before_present{ false };
-    VkDebugUtilsMessageSeverityFlagBitsEXT debug_message_severity{ VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT };
+    VkFlags                      debug_message_severity{ VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT |
+                                    VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT };
 
     // Dumping resources related configurable replay options
-    std::vector<uint64_t>                           BeginCommandBuffer_Indices;
-    std::vector<std::vector<uint64_t>>              Draw_Indices;
-    std::vector<std::vector<std::vector<uint64_t>>> RenderPass_Indices;
-    std::vector<std::vector<uint64_t>>              Dispatch_Indices;
-    std::vector<std::vector<uint64_t>>              TraceRays_Indices;
-    std::vector<uint64_t>                           QueueSubmit_Indices;
-    std::string                                     dump_resources_block_indices;
-    util::ScreenshotFormat                          dump_resources_image_format{ util::ScreenshotFormat::kBmp };
+    std::vector<BeginCmdBufQueueSubmitPair> BeginCommandBufferQueueSubmit_Indices;
+
+    std::vector<RenderPassIndices> RenderPass_Indices;
+    std::vector<CommandIndices>    Draw_Indices;
+    CommandImageSubresource        DrawSubresources;
+
+    std::vector<CommandIndices> Dispatch_Indices;
+    CommandImageSubresource     DispatchSubresources;
+
+    std::vector<CommandIndices> TraceRays_Indices;
+    CommandImageSubresource     TraceRaysSubresources;
+
+    std::vector<CommandIndices> Transfer_Indices;
+    CommandImageSubresource     TransferSubresources;
+
+    // ExecuteCommands block index : vector or BeginCommandBuffer indices of secondary cbs.
+    std::vector<ExecuteCommands> ExecuteCommands_Indices;
+
+    std::string            dump_resources_block_indices;
+    util::ScreenshotFormat dump_resources_image_format{ util::ScreenshotFormat::kBmp };
 
     // Flag to quickly check whether the feature is enabled or not
     bool  dumping_resources{ false };
@@ -90,10 +150,14 @@ struct VulkanReplayOptions : public ReplayOptions
     float dump_resources_scale{ 1.0f };
     bool  dump_resources_dump_vertex_index_buffer{ false };
     bool  dump_resources_json_per_command{ false };
-    bool  dump_resources_dump_immutable_resources{ false };
+    bool  dump_all_descriptors{ false };
     bool  dump_resources_dump_all_image_subresources{ false };
     bool  dump_resources_dump_raw_images{ false };
     bool  dump_resources_dump_separate_alpha{ false };
+    bool  dump_resources_dump_unused_vertex_bindings{ false };
+    bool  dump_resources_dump_build_AS_input_buffers{ false };
+
+    format::CompressionType dump_resources_binary_file_compression_type{ format::CompressionType::kNone };
 
     bool preload_measurement_range{ false };
 
