@@ -27,6 +27,7 @@
 #include "decode/decoder_util.h"
 #include "graphics/vulkan_util.h"
 #include "util/platform.h"
+#include "util/alignment_utils.h"
 
 #include "Vulkan-Utility-Libraries/vk_format_utils.h"
 
@@ -58,7 +59,7 @@ static uint32_t FindBufferOffsetAlignmentForCopyBufferToImage(VkFormat format)
     }
     else
     {
-        // In all othe cases spec mandates an alignment of the format's block size.
+        // In all other cases spec mandates an alignment of the format's block size.
         const VKU_FORMAT_INFO format_info = vkuGetFormatInfo(format);
         alignment                         = format_info.block_size;
     }
@@ -67,30 +68,50 @@ static uint32_t FindBufferOffsetAlignmentForCopyBufferToImage(VkFormat format)
 }
 
 VulkanResourceInitializer::VulkanResourceInitializer(const VulkanDeviceInfo*                 device_info,
+                                                     VkDeviceSize                            total_copy_size,
                                                      VkDeviceSize                            max_copy_size,
+                                                     const VkPhysicalDeviceProperties&       physical_device_properties,
                                                      const VkPhysicalDeviceMemoryProperties& memory_properties,
                                                      bool                                    have_shader_stencil_write,
                                                      VulkanResourceAllocator*                resource_allocator,
                                                      const graphics::VulkanDeviceTable*      device_table) :
     device_(device_info->handle),
     staging_memory_(VK_NULL_HANDLE), staging_memory_data_(0), staging_buffer_(VK_NULL_HANDLE), staging_buffer_data_(0),
-    staging_buffer_mapped_ptr_(nullptr), staging_buffer_offset_(0), draw_sampler_(VK_NULL_HANDLE),
-    draw_pool_(VK_NULL_HANDLE), draw_set_layout_(VK_NULL_HANDLE), draw_set_(VK_NULL_HANDLE),
-    max_copy_size_(max_copy_size), have_shader_stencil_write_(have_shader_stencil_write),
-    resource_allocator_(resource_allocator), device_table_(device_table), device_info_(device_info)
+    staging_buffer_mapped_ptr_(nullptr), staging_buffer_offset_(0), staging_buffer_size_(0),
+    draw_sampler_(VK_NULL_HANDLE), draw_pool_(VK_NULL_HANDLE), draw_set_layout_(VK_NULL_HANDLE),
+    draw_set_(VK_NULL_HANDLE), memory_properties_(memory_properties),
+    have_shader_stencil_write_(have_shader_stencil_write), resource_allocator_(resource_allocator),
+    device_table_(device_table), device_info_(device_info)
 {
-    assert((device_info != nullptr) && (device_info->handle != VK_NULL_HANDLE) &&
-           (memory_properties.memoryTypeCount > 0) && (memory_properties.memoryHeapCount > 0) &&
-           (resource_allocator != nullptr) && (device_table != nullptr));
+    GFXRECON_ASSERT((device_info != nullptr) && (device_info->handle != VK_NULL_HANDLE) &&
+                    (memory_properties.memoryTypeCount > 0) && (memory_properties.memoryHeapCount > 0) &&
+                    (resource_allocator != nullptr) && (device_table != nullptr));
 
-    size_t type_size = memory_properties.memoryTypeCount * sizeof(memory_properties.memoryTypes[0]);
-    size_t heap_size = memory_properties.memoryHeapCount * sizeof(memory_properties.memoryHeaps[0]);
+    // flushes of staging-buffer need to be aligned to nonCoherentAtomSize
+    staging_buffer_alignment_ = physical_device_properties.limits.nonCoherentAtomSize;
 
-    memory_properties_.memoryTypeCount = memory_properties.memoryTypeCount;
-    memory_properties_.memoryHeapCount = memory_properties.memoryHeapCount;
+    memory_properties_ = memory_properties;
 
-    util::platform::MemoryCopy(&memory_properties_.memoryTypes, type_size, &memory_properties.memoryTypes, type_size);
-    util::platform::MemoryCopy(&memory_properties_.memoryHeaps, heap_size, &memory_properties.memoryHeaps, heap_size);
+    // determine sane size for the staging-buffer, set boundary to max(max_copy, 128Mb)
+    // set a minimum size, used for captures that do not report 'total_copy_size'  (total_copy_size == max_copy_size)
+    VkDeviceSize staging_buffer_min = std::max<VkDeviceSize>(max_copy_size, 32U << 20U);
+    VkDeviceSize staging_buffer_max = std::max<VkDeviceSize>(staging_buffer_min, 128U << 20U);
+    staging_buffer_size_            = std::clamp(total_copy_size, staging_buffer_min, staging_buffer_max);
+    staging_buffer_size_            = util::aligned_value(staging_buffer_size_, staging_buffer_alignment_);
+
+    VkFlags staging_mem_flags = VK_MEMORY_PROPERTY_HOST_COHERENT_BIT | VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT;
+    auto    mem_index         = GetMemoryTypeIndex(0xFFFFFFFF, staging_mem_flags);
+    if (!mem_index ||
+        memory_properties.memoryHeaps[memory_properties.memoryTypes[*mem_index].heapIndex].size < staging_buffer_size_)
+    {
+        GFXRECON_LOG_WARNING("%s: no suitable staging-buffer could be created", __func__);
+    }
+
+    constexpr VkFenceCreateInfo fence_ci = { VK_STRUCTURE_TYPE_FENCE_CREATE_INFO,
+                                             nullptr,
+                                             static_cast<VkFenceCreateFlags>(0) };
+    VkResult                    result   = device_table_->CreateFence(device_, &fence_ci, nullptr, &fence_);
+    GFXRECON_ASSERT(result == VK_SUCCESS);
 }
 
 VulkanResourceInitializer::~VulkanResourceInitializer()
@@ -117,6 +138,11 @@ VulkanResourceInitializer::~VulkanResourceInitializer()
     {
         device_table_->DestroyDescriptorSetLayout(device_, draw_set_layout_, nullptr);
     }
+
+    if (fence_ != VK_NULL_HANDLE)
+    {
+        device_table_->DestroyFence(device_, fence_, nullptr);
+    }
 }
 
 VkResult VulkanResourceInitializer::LoadData(VkDeviceSize                          size,
@@ -152,7 +178,7 @@ VkResult VulkanResourceInitializer::InitializeBuffer(VkDeviceSize        data_si
     if (result == VK_SUCCESS)
     {
         // No space left in staging buffer
-        if (staging_buffer_offset_ + data_size > max_copy_size_)
+        if (staging_buffer_offset_ + data_size > staging_buffer_size_)
         {
             FlushStagingBuffer();
             result = FlushCommandBuffer(queue_family_index);
@@ -181,15 +207,12 @@ VkResult VulkanResourceInitializer::InitializeBuffer(VkDeviceSize        data_si
                     offsetted_regions_copy_[i].srcOffset += staging_buffer_offset_;
                 }
 
-                if (result == VK_SUCCESS)
-                {
-                    device_table_->CmdCopyBuffer(
-                        command_buffer, staging_buffer_, buffer, region_count, offsetted_regions_copy_.data());
+                device_table_->CmdCopyBuffer(
+                    command_buffer, staging_buffer_, buffer, region_count, offsetted_regions_copy_.data());
 
-                    // Advance staging buffer offset
-                    GFXRECON_ASSERT(staging_buffer_offset_ + data_size <= max_copy_size_);
-                    staging_buffer_offset_ = staging_buffer_offset_ + data_size;
-                }
+                // Advance staging buffer offset
+                GFXRECON_ASSERT(staging_buffer_offset_ + data_size <= staging_buffer_size_);
+                staging_buffer_offset_ = staging_buffer_offset_ + data_size;
             }
         }
     }
@@ -264,7 +287,7 @@ VkResult VulkanResourceInitializer::InitializeImage(VkDeviceSize             dat
 
                 // If data does not fit in the remaining portion of the staging buffer flush pending commands and reset
                 // staging buffer offset
-                if (staging_buffer_offset_ + data_size > max_copy_size_)
+                if (staging_buffer_offset_ + data_size > staging_buffer_size_)
                 {
                     FlushStagingBuffer();
                     result = FlushCommandBuffer(queue_family_index);
@@ -314,7 +337,7 @@ VkResult VulkanResourceInitializer::InitializeImage(VkDeviceSize             dat
                                        offsetted_level_copies_.data());
 
             // Advance staging buffer offset
-            GFXRECON_ASSERT(staging_buffer_offset_ + data_size <= max_copy_size_);
+            GFXRECON_ASSERT(staging_buffer_offset_ + data_size <= staging_buffer_size_);
             staging_buffer_offset_ = staging_buffer_offset_ + data_size;
         }
     }
@@ -365,7 +388,12 @@ VkResult VulkanResourceInitializer::TransitionImage(uint32_t              queue_
                                           1,
                                           &memory_barrier);
 
-        result = FlushCommandBuffer(queue_family_index);
+        // Check if there are pending commands from InitializeImage by checking the staging buffer's offset.
+        // If there are then don't submit the command buffer as there is still room left to init more resources
+        if (!staging_buffer_offset_)
+        {
+            result = FlushCommandBuffer(queue_family_index);
+        }
     }
 
     return result;
@@ -897,14 +925,13 @@ VkResult VulkanResourceInitializer::CreateStagingImage(const VkImageCreateInfo*
 
         device_table_->GetImageMemoryRequirements(device_, staging_image, &memory_reqs);
 
-        uint32_t memory_type_index =
-            GetMemoryTypeIndex(memory_reqs.memoryTypeBits, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+        auto memory_type_index = GetMemoryTypeIndex(memory_reqs.memoryTypeBits, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 
-        assert(memory_type_index != std::numeric_limits<uint32_t>::max());
+        GFXRECON_ASSERT(memory_type_index);
 
         VkMemoryAllocateInfo alloc_info = { VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO };
         alloc_info.pNext                = nullptr;
-        alloc_info.memoryTypeIndex      = memory_type_index;
+        alloc_info.memoryTypeIndex      = *memory_type_index;
         alloc_info.allocationSize       = memory_reqs.size;
 
         result = resource_allocator_->AllocateMemoryDirect(&alloc_info, nullptr, &staging_memory, &staging_memory_data);
@@ -997,7 +1024,11 @@ VkResult VulkanResourceInitializer::AcquireStagingBuffer(VkDeviceSize size)
 {
     VkResult result = VK_SUCCESS;
 
-    if ((staging_buffer_ == VK_NULL_HANDLE) || (size > max_copy_size_))
+    // increase size if necessary, but we expect this is not necessary
+    GFXRECON_ASSERT(size <= staging_buffer_size_);
+    size = std::max<VkDeviceSize>(util::aligned_value(size, staging_buffer_alignment_), staging_buffer_size_);
+
+    if (staging_buffer_ == VK_NULL_HANDLE || size > staging_buffer_size_)
     {
         if (staging_buffer_ != VK_NULL_HANDLE)
         {
@@ -1008,7 +1039,7 @@ VkResult VulkanResourceInitializer::AcquireStagingBuffer(VkDeviceSize size)
         VkBufferCreateInfo create_info    = { VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO };
         create_info.pNext                 = nullptr;
         create_info.flags                 = 0;
-        create_info.size                  = std::max(size, max_copy_size_);
+        create_info.size                  = std::max<VkDeviceSize>(size, staging_buffer_size_);
         create_info.usage                 = VK_BUFFER_USAGE_TRANSFER_SRC_BIT;
         create_info.sharingMode           = VK_SHARING_MODE_EXCLUSIVE;
         create_info.queueFamilyIndexCount = 0;
@@ -1022,15 +1053,15 @@ VkResult VulkanResourceInitializer::AcquireStagingBuffer(VkDeviceSize size)
             VkMemoryRequirements memory_requirements;
             device_table_->GetBufferMemoryRequirements(device_, staging_buffer_, &memory_requirements);
 
-            uint32_t memory_type_index =
+            auto memory_type_index =
                 GetMemoryTypeIndex(memory_requirements.memoryTypeBits, VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT);
 
-            assert(memory_type_index != std::numeric_limits<uint32_t>::max());
+            GFXRECON_ASSERT(memory_type_index);
 
             VkMemoryAllocateInfo alloc_info = { VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO };
             alloc_info.pNext                = nullptr;
             alloc_info.allocationSize       = memory_requirements.size;
-            alloc_info.memoryTypeIndex      = memory_type_index;
+            alloc_info.memoryTypeIndex      = *memory_type_index;
 
             // Allocate the memory for the buffer.
             result = resource_allocator_->AllocateMemoryDirect(
@@ -1045,11 +1076,14 @@ VkResult VulkanResourceInitializer::AcquireStagingBuffer(VkDeviceSize size)
 
             if (result == VK_SUCCESS)
             {
-                max_copy_size_ = size;
+                staging_buffer_size_ = size;
 
                 // Map staging buffer
-                result = resource_allocator_->MapResourceMemoryDirect(
-                    max_copy_size_, 0, reinterpret_cast<void**>(&staging_buffer_mapped_ptr_), staging_buffer_data_);
+                result =
+                    resource_allocator_->MapResourceMemoryDirect(staging_buffer_size_,
+                                                                 0,
+                                                                 reinterpret_cast<void**>(&staging_buffer_mapped_ptr_),
+                                                                 staging_buffer_data_);
             }
             else
             {
@@ -1062,7 +1096,6 @@ VkResult VulkanResourceInitializer::AcquireStagingBuffer(VkDeviceSize size)
             }
         }
     }
-
     return result;
 }
 
@@ -1164,13 +1197,25 @@ VkResult VulkanResourceInitializer::ExecuteCommandBuffer(VkQueue queue, VkComman
     submit_info.signalSemaphoreCount = 0;
     submit_info.pSignalSemaphores    = nullptr;
 
-    VkResult result = device_table_->QueueSubmit(queue, 1, &submit_info, VK_NULL_HANDLE);
-
-    if (result == VK_SUCCESS)
+    VkResult result = device_table_->QueueSubmit(queue, 1, &submit_info, fence_);
+    if (result != VK_SUCCESS)
     {
-        result = device_table_->QueueWaitIdle(queue);
+        return result;
     }
 
+    // keep track of queue-submits for introspection and profiling
+    num_queue_submits_++;
+
+    // Wait a sensible amount of time (10 seconds) to avoid hanging in case a prior
+    // operation caused the GPU to hang or crash.
+    result = device_table_->WaitForFences(device_, 1, &fence_, VK_TRUE, 10000000000);
+    if (result != VK_SUCCESS)
+    {
+        return result;
+    }
+
+    // reset to unsignaled state
+    result = device_table_->ResetFences(device_, 1, &fence_);
     return result;
 }
 
@@ -1201,9 +1246,10 @@ VkImageAspectFlags VulkanResourceInitializer::GetImageTransitionAspect(VkFormat
     return transition_aspect;
 }
 
-uint32_t VulkanResourceInitializer::GetMemoryTypeIndex(uint32_t type_bits, VkMemoryPropertyFlags property_flags)
+std::optional<uint32_t> VulkanResourceInitializer::GetMemoryTypeIndex(uint32_t              type_bits,
+                                                                      VkMemoryPropertyFlags property_flags) const
 {
-    uint32_t memory_type_index = std::numeric_limits<uint32_t>::max();
+    std::optional<uint32_t> memory_type_index;
 
     for (uint32_t i = 0; i < memory_properties_.memoryTypeCount; ++i)
     {
@@ -1214,7 +1260,6 @@ uint32_t VulkanResourceInitializer::GetMemoryTypeIndex(uint32_t type_bits, VkMem
             break;
         }
     }
-
     return memory_type_index;
 }
 
@@ -1500,11 +1545,8 @@ VkResult VulkanResourceInitializer::FlushCommandBuffer(uint32_t queue_family_ind
     {
         device_table_->EndCommandBuffer(iter->second.command_buffer);
 
-        result = ExecuteCommandBuffer(iter->second.queue, iter->second.command_buffer);
-        if (result == VK_SUCCESS)
-        {
-            iter->second.recording = false;
-        }
+        result                 = ExecuteCommandBuffer(iter->second.queue, iter->second.command_buffer);
+        iter->second.recording = false;
     }
 
     return result;
@@ -1539,7 +1581,8 @@ void VulkanResourceInitializer::FlushStagingBuffer()
         memory_range.pNext  = nullptr;
         memory_range.memory = staging_memory_;
         memory_range.offset = 0;
-        memory_range.size   = staging_buffer_offset_;
+        memory_range.size   = std::min<VkDeviceSize>(
+            staging_buffer_size_, util::aligned_value(staging_buffer_offset_, staging_buffer_alignment_));
         resource_allocator_->FlushMappedMemoryRangesDirect(1, &memory_range, &staging_memory_data_);
 
         staging_buffer_offset_ = 0;
