@@ -25,6 +25,7 @@
 #include "decode/vulkan_resource_allocator.h"
 #include "decode/decoder_util.h"
 #include "decode/mark_injected_commands.h"
+#include "graphics/vulkan_resources_util.h"
 #include "vulkan/vulkan_core.h"
 #include <array>
 
@@ -51,15 +52,16 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainKHR(VkResult
                                                     HandlePointerDecoder<VkSwapchainKHR>* swapchain,
                                                     const graphics::VulkanDeviceTable*    device_table)
 {
-    VkDevice                 device = VK_NULL_HANDLE;
+    VkDevice                 device          = VK_NULL_HANDLE;
+    VkPhysicalDevice         physical_device = VK_NULL_HANDLE;
     VkSurfaceCapabilitiesKHR surfCapabilities{};
 
     if (device_info != nullptr)
     {
         device = device_info->handle;
+        physical_device = device_info->parent;
     }
-    device_table_                    = device_table;
-    VkPhysicalDevice physical_device = device_info->parent;
+    device_table_ = device_table;
 
     VkSwapchainCreateInfoKHR modified_create_info = *create_info;
     modified_create_info.imageUsage =
@@ -216,12 +218,13 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainResourceData(const VulkanDeviceI
     {
         // If we're past the point of enabled queues, then stop looking because we really can't enable
         // a queue that isn't flagged during device creation.
-        if (queue_family_index >= static_cast<uint32_t>(device_info->queue_family_index_enabled.size()))
+        if (queue_family_index >=
+            static_cast<uint32_t>(device_info->enabled_queue_family_flags.queue_family_index_enabled.size()))
         {
             break;
         }
 
-        if (!device_info->queue_family_index_enabled[queue_family_index])
+        if (!device_info->enabled_queue_family_flags.queue_family_index_enabled[queue_family_index])
         {
             continue;
         }
@@ -277,8 +280,8 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainResourceData(const VulkanDeviceI
                 // We only want to look at a given queue if it was enabled during device creation time
                 // and if it supports present.  Otherwise, we don't need to create a command pool,
                 // command buffers, and semaphores for performing the swapchain copy.
-                if (device_info->queue_family_index_enabled.size() <= queue_family_index ||
-                    !device_info->queue_family_index_enabled[queue_family_index])
+                if (device_info->enabled_queue_family_flags.queue_family_index_enabled.size() <= queue_family_index ||
+                    !device_info->enabled_queue_family_flags.queue_family_index_enabled[queue_family_index])
                 {
                     GFXRECON_LOG_DEBUG("Virtual swapchain skipping creating blit info for queue family %d because it "
                                        "was not enabled by the device",
@@ -327,86 +330,81 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainResourceData(const VulkanDeviceI
             // Command Buffers, Semaphores, etc) as many queue families that are available.
             // This is because at any point, the application may get a Device queue from that family and
             // use it during the present.
-            uint32_t start_size = static_cast<uint32_t>(copy_cmd_data.command_buffers.size());
-            uint32_t new_count  = property_count;
-            if (start_size < new_count)
+            // Create one command buffer per queue per swapchain image so that we don't reset a command buffer that
+            // may be in active use.
+            uint32_t command_buffer_count = static_cast<uint32_t>(copy_cmd_data.command_buffers.size());
+            if (command_buffer_count < capture_image_count)
             {
-                // Create one command buffer per queue per swapchain image so that we don't reset a command buffer that
-                // may be in active use.
-                uint32_t command_buffer_count = static_cast<uint32_t>(copy_cmd_data.command_buffers.size());
-                if (command_buffer_count < capture_image_count)
-                {
-                    copy_cmd_data.command_buffers.resize(capture_image_count);
+                copy_cmd_data.command_buffers.resize(capture_image_count);
 
-                    uint32_t                    new_count     = capture_image_count - command_buffer_count;
-                    VkCommandBufferAllocateInfo allocate_info = {
-                        VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO, // sType
-                        nullptr,                                        // pNext
-                        copy_cmd_data.command_pool,                     // commandPool
-                        VK_COMMAND_BUFFER_LEVEL_PRIMARY,                // level
-                        new_count                                       // commandBufferCount
+                uint32_t                    new_count     = capture_image_count - command_buffer_count;
+                VkCommandBufferAllocateInfo allocate_info = {
+                    VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO, // sType
+                    nullptr,                                        // pNext
+                    copy_cmd_data.command_pool,                     // commandPool
+                    VK_COMMAND_BUFFER_LEVEL_PRIMARY,                // level
+                    new_count                                       // commandBufferCount
+                };
+
+                result = device_table_->AllocateCommandBuffers(
+                    device, &allocate_info, &copy_cmd_data.command_buffers[command_buffer_count]);
+                if (result != VK_SUCCESS)
+                {
+                    GFXRECON_LOG_ERROR("Virtual swapchain failed allocating internal command buffer %d for "
+                                       "swapchain (ID = %" PRIu64 ")",
+                                       queue_family_index,
+                                       swapchain_info->capture_id);
+                    return result;
+                }
+            }
+            uint32_t semaphore_count = static_cast<uint32_t>(copy_cmd_data.semaphores.size());
+            if (semaphore_count < capture_image_count)
+            {
+                copy_cmd_data.semaphores.resize(capture_image_count);
+
+                for (uint32_t ii = semaphore_count; ii < capture_image_count; ++ii)
+                {
+                    VkSemaphoreCreateInfo semaphore_create_info = {
+                        VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO, // sType
+                        nullptr,                                 // pNext
+                        0                                        // flags
                     };
 
-                    result = device_table_->AllocateCommandBuffers(
-                        device, &allocate_info, &copy_cmd_data.command_buffers[command_buffer_count]);
+                    VkSemaphore semaphore = 0;
+                    result = device_table_->CreateSemaphore(device, &semaphore_create_info, nullptr, &semaphore);
                     if (result != VK_SUCCESS)
                     {
-                        GFXRECON_LOG_ERROR("Virtual swapchain failed allocating internal command buffer %d for "
+                        GFXRECON_LOG_ERROR("Virtual swapchain failed creating internal copy semaphore for "
                                            "swapchain (ID = %" PRIu64 ")",
-                                           queue_family_index,
                                            swapchain_info->capture_id);
                         return result;
                     }
+                    copy_cmd_data.semaphores[ii] = semaphore;
                 }
-                uint32_t semaphore_count = static_cast<uint32_t>(copy_cmd_data.semaphores.size());
-                if (semaphore_count < capture_image_count)
+            }
+            uint32_t fence_count = static_cast<uint32_t>(copy_cmd_data.fences.size());
+            if (fence_count < capture_image_count)
+            {
+                copy_cmd_data.fences.resize(capture_image_count);
+
+                for (uint32_t ii = fence_count; ii < capture_image_count; ++ii)
                 {
-                    copy_cmd_data.semaphores.resize(capture_image_count);
+                    VkFenceCreateInfo fence_create_info = {
+                        VK_STRUCTURE_TYPE_FENCE_CREATE_INFO, // sType
+                        nullptr,                             // pNext
+                        VK_FENCE_CREATE_SIGNALED_BIT         // flags
+                    };
 
-                    for (uint32_t ii = semaphore_count; ii < capture_image_count; ++ii)
+                    VkFence fence = VK_NULL_HANDLE;
+                    result        = device_table_->CreateFence(device, &fence_create_info, nullptr, &fence);
+                    if (result != VK_SUCCESS)
                     {
-                        VkSemaphoreCreateInfo semaphore_create_info = {
-                            VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO, // sType
-                            nullptr,                                 // pNext
-                            0                                        // flags
-                        };
-
-                        VkSemaphore semaphore = 0;
-                        result = device_table_->CreateSemaphore(device, &semaphore_create_info, nullptr, &semaphore);
-                        if (result != VK_SUCCESS)
-                        {
-                            GFXRECON_LOG_ERROR("Virtual swapchain failed creating internal copy semaphore for "
-                                               "swapchain (ID = %" PRIu64 ")",
-                                               swapchain_info->capture_id);
-                            return result;
-                        }
-                        copy_cmd_data.semaphores[ii] = semaphore;
-                    }
-                }
-                uint32_t fence_count = static_cast<uint32_t>(copy_cmd_data.fences.size());
-                if (fence_count < capture_image_count)
-                {
-                    copy_cmd_data.fences.resize(capture_image_count);
-
-                    for (uint32_t ii = fence_count; ii < capture_image_count; ++ii)
-                    {
-                        VkFenceCreateInfo fence_create_info = {
-                            VK_STRUCTURE_TYPE_FENCE_CREATE_INFO, // sType
-                            nullptr,                             // pNext
-                            VK_FENCE_CREATE_SIGNALED_BIT         // flags
-                        };
-
-                        VkFence fence = VK_NULL_HANDLE;
-                        result        = device_table_->CreateFence(device, &fence_create_info, nullptr, &fence);
-                        if (result != VK_SUCCESS)
-                        {
-                            GFXRECON_LOG_ERROR("Virtual swapchain failed creating internal copy fence for "
-                                               "swapchain (ID = %" PRIu64 ")",
-                                               swapchain_info->capture_id);
-                            return result;
-                        }
-                        copy_cmd_data.fences[ii] = fence;
+                        GFXRECON_LOG_ERROR("Virtual swapchain failed creating internal copy fence for "
+                                           "swapchain (ID = %" PRIu64 ")",
+                                           swapchain_info->capture_id);
+                        return result;
                     }
+                    copy_cmd_data.fences[ii] = fence;
                 }
             }
         }
@@ -520,7 +518,7 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainResourceData(const VulkanDeviceI
                 VK_QUEUE_FAMILY_IGNORED,                // dstQueueFamilyIndex
                 VK_NULL_HANDLE,                         // image
                 VkImageSubresourceRange{
-                    VK_IMAGE_ASPECT_COLOR_BIT,
+                    graphics::GetFormatAspects(swapchain_info->format),
                     0,
                     image_create_info.mipLevels,
                     0,
@@ -808,6 +806,13 @@ VkResult VulkanVirtualSwapchain::QueuePresentKHR(VkResult
         uint32_t    capture_image_index = capture_image_indices[i];
         uint32_t    replay_image_index  = present_info->pImageIndices[i];
 
+        auto aspect_mask       = graphics::GetFormatAspects(swapchain_info->format);
+        subresource.aspectMask = aspect_mask;
+        initial_barrier_virtual_image.subresourceRange.aspectMask   = aspect_mask;
+        final_barrier_virtual_image.subresourceRange.aspectMask     = aspect_mask;
+        initial_barrier_swapchain_image.subresourceRange.aspectMask = aspect_mask;
+        final_barrier_swapchain_image.subresourceRange.aspectMask   = aspect_mask;
+
         // Get the per swapchain resource data so we have access to the virtual swapchain-specific information.
         if (swapchain_resources_.find(swapchain_info->handle) == swapchain_resources_.end())
         {
