@@ -510,7 +510,7 @@ VKAPI_ATTR VkResult VKAPI_CALL vkCreateGraphicsPipelines(VkDevice
                 }
                 VulkanCaptureManager::Get()->WriteAnnotation(
                     format::AnnotationType::kText, format::kAnnotationPipelineCreationAttempt, "");
-                GFXRECON_LOG_WARNING(
+                GFXRECON_LOG_WARNING_ONCE(
                     "VK_PIPELINE_CREATE_FAIL_ON_PIPELINE_COMPILE_REQUIRED_BIT isn't suppported. Skip dispatch "
                     "CreateGraphicsPipelines and not record the call. Force to return VK_PIPELINE_COMPILE_REQUIRED.");
                 return VK_PIPELINE_COMPILE_REQUIRED;
@@ -605,7 +605,7 @@ VKAPI_ATTR VkResult VKAPI_CALL vkCreateComputePipelines(VkDevice
                 }
                 VulkanCaptureManager::Get()->WriteAnnotation(
                     format::AnnotationType::kText, format::kAnnotationPipelineCreationAttempt, "");
-                GFXRECON_LOG_WARNING(
+                GFXRECON_LOG_WARNING_ONCE(
                     "VK_PIPELINE_CREATE_FAIL_ON_PIPELINE_COMPILE_REQUIRED_BIT isn't suppported. Skip dispatch "
                     "CreateComputePipelines and not record the call. Force to return VK_PIPELINE_COMPILE_REQUIRED.");
                 return VK_PIPELINE_COMPILE_REQUIRED;
@@ -700,7 +700,7 @@ VKAPI_ATTR VkResult VKAPI_CALL vkCreateRayTracingPipelinesNV(VkDevice
                 }
                 VulkanCaptureManager::Get()->WriteAnnotation(
                     format::AnnotationType::kText, format::kAnnotationPipelineCreationAttempt, "");
-                GFXRECON_LOG_WARNING(
+                GFXRECON_LOG_WARNING_ONCE(
                     "VK_PIPELINE_CREATE_FAIL_ON_PIPELINE_COMPILE_REQUIRED_BIT isn't suppported. Skip dispatch "
                     "CreateRayTracingPipelinesNV and not record the call. Force to return "
                     "VK_PIPELINE_COMPILE_REQUIRED.");
@@ -798,7 +798,7 @@ VKAPI_ATTR VkResult VKAPI_CALL vkCreateRayTracingPipelinesKHR(VkDevice
                 }
                 VulkanCaptureManager::Get()->WriteAnnotation(
                     format::AnnotationType::kText, format::kAnnotationPipelineCreationAttempt, "");
-                GFXRECON_LOG_WARNING(
+                GFXRECON_LOG_WARNING_ONCE(
                     "VK_PIPELINE_CREATE_FAIL_ON_PIPELINE_COMPILE_REQUIRED_BIT isn't suppported. Skip dispatch "
                     "CreateRayTracingPipelinesKHR and not record the call. Force to return "
                     "VK_PIPELINE_COMPILE_REQUIRED.");
@@ -895,5 +895,60 @@ VKAPI_ATTR VkResult VKAPI_CALL vkCreateRayTracingPipelinesKHR(VkDevice
     return result;
 }
 
+VKAPI_ATTR VkResult VKAPI_CALL vkGetAccelerationStructureOpaqueCaptureDescriptorDataEXT(
+    VkDevice device, const VkAccelerationStructureCaptureDescriptorDataInfoEXT* pInfo, void* pData)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(device);
+    GFXRECON_UNREFERENCED_PARAMETER(pInfo);
+    GFXRECON_UNREFERENCED_PARAMETER(pData);
+    GFXRECON_LOG_WARNING_ONCE(
+        "%s: skipping capture of VkPhysicalDeviceDescriptorBufferFeaturesEXT::descriptorBufferCaptureReplay", __func__);
+    return VK_SUCCESS;
+}
+
+VKAPI_ATTR VkResult VKAPI_CALL vkGetBufferOpaqueCaptureDescriptorDataEXT(
+    VkDevice device, const VkBufferCaptureDescriptorDataInfoEXT* pInfo, void* pData)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(device);
+    GFXRECON_UNREFERENCED_PARAMETER(pInfo);
+    GFXRECON_UNREFERENCED_PARAMETER(pData);
+    GFXRECON_LOG_WARNING_ONCE(
+        "%s: skipping capture of VkPhysicalDeviceDescriptorBufferFeaturesEXT::descriptorBufferCaptureReplay", __func__);
+    return VK_SUCCESS;
+}
+
+VKAPI_ATTR VkResult VKAPI_CALL
+vkGetImageOpaqueCaptureDescriptorDataEXT(VkDevice device, const VkImageCaptureDescriptorDataInfoEXT* pInfo, void* pData)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(device);
+    GFXRECON_UNREFERENCED_PARAMETER(pInfo);
+    GFXRECON_UNREFERENCED_PARAMETER(pData);
+    GFXRECON_LOG_WARNING_ONCE(
+        "%s: skipping capture of VkPhysicalDeviceDescriptorBufferFeaturesEXT::descriptorBufferCaptureReplay", __func__);
+    return VK_SUCCESS;
+}
+
+VKAPI_ATTR VkResult VKAPI_CALL vkGetSamplerOpaqueCaptureDescriptorDataEXT(
+    VkDevice device, const VkSamplerCaptureDescriptorDataInfoEXT* pInfo, void* pData)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(device);
+    GFXRECON_UNREFERENCED_PARAMETER(pInfo);
+    GFXRECON_UNREFERENCED_PARAMETER(pData);
+    GFXRECON_LOG_WARNING_ONCE(
+        "%s: skipping capture of VkPhysicalDeviceDescriptorBufferFeaturesEXT::descriptorBufferCaptureReplay", __func__);
+    return VK_SUCCESS;
+}
+
+VKAPI_ATTR VkResult VKAPI_CALL vkGetImageViewOpaqueCaptureDescriptorDataEXT(
+    VkDevice device, const VkImageViewCaptureDescriptorDataInfoEXT* pInfo, void* pData)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(device);
+    GFXRECON_UNREFERENCED_PARAMETER(pInfo);
+    GFXRECON_UNREFERENCED_PARAMETER(pData);
+    GFXRECON_LOG_WARNING_ONCE(
+        "%s: skipping capture of VkPhysicalDeviceDescriptorBufferFeaturesEXT::descriptorBufferCaptureReplay", __func__);
+    return VK_SUCCESS;
+}
+
 GFXRECON_END_NAMESPACE(encode)
 GFXRECON_END_NAMESPACE(gfxrecon)
