@@ -27,11 +27,13 @@
 #include "encode/descriptor_update_template_info.h"
 #include "encode/vulkan_state_info.h"
 #include "encode/handle_unwrap_memory.h"
+#include "encode/vulkan_acceleration_structure_build_state.h"
 #include "format/format.h"
 #include "generated/generated_vulkan_dispatch_table.h"
 #include "graphics/vulkan_util.h"
 #include "graphics/vulkan_device_util.h"
 #include "graphics/vulkan_instance_util.h"
+#include "graphics/vulkan_resources_util.h"
 #include "util/defines.h"
 #include "util/memory_output_stream.h"
 #include "util/page_guard_manager.h"
@@ -46,6 +48,7 @@
 #include <map>
 #include <unordered_map>
 #include <unordered_set>
+#include <variant>
 #include <vector>
 #include <optional>
 
@@ -144,9 +147,10 @@ struct PhysicalDeviceWrapper : public HandleWrapper<VkPhysicalDevice>
     std::unique_ptr<VkQueueFamilyProperties2[]> queue_family_properties2;
     std::vector<std::unique_ptr<VkQueueFamilyCheckpointPropertiesNV>> queue_family_checkpoint_properties;
 
-    // Track RayTracingPipeline / AccelerationStructure properties
+    // track properties of various physical-device extensions
     std::optional<VkPhysicalDeviceRayTracingPipelinePropertiesKHR>    ray_tracing_pipeline_properties;
     std::optional<VkPhysicalDeviceAccelerationStructurePropertiesKHR> acceleration_structure_properties;
+    std::optional<VkPhysicalDeviceDescriptorBufferPropertiesEXT>      descriptor_buffer_properties;
 };
 
 struct InstanceWrapper : public HandleWrapper<VkInstance>
@@ -165,8 +169,8 @@ struct QueueWrapper : public HandleWrapper<VkQueue>
 struct DeviceWrapper : public HandleWrapper<VkDevice>
 {
     graphics::VulkanDeviceTable layer_table;
-    PhysicalDeviceWrapper*     physical_device{ nullptr };
-    std::vector<QueueWrapper*> child_queues;
+    PhysicalDeviceWrapper*      physical_device{ nullptr };
+    std::vector<QueueWrapper*>  child_queues;
 
     // Physical device property & feature state at device creation
     graphics::VulkanDevicePropertyFeatureInfo              property_feature_info;
@@ -178,8 +182,11 @@ struct FenceWrapper : public HandleWrapper<VkFence>
 {
     // Signaled state at creation to be compared with signaled state at snapshot write. If states are different, the
     // create parameters will need to be modified to reflect the state at snapshot write.
-    bool           created_signaled{ false };
-    DeviceWrapper* device{ nullptr };
+    bool created_signaled{ false };
+
+    // keep track if the fence is currently waiting to be signaled
+    std::atomic<bool> in_flight{ false };
+    DeviceWrapper*    device{ nullptr };
 };
 
 struct EventWrapper : public HandleWrapper<VkEvent>
@@ -207,12 +214,23 @@ struct BufferViewWrapper;
 struct BufferWrapper : public HandleWrapper<VkBuffer>, AssetWrapperBase
 {
     // State tracking info for buffers with device addresses.
-    format::HandleId   device_id{ format::kNullHandleId };
+    VkDevice           device{ VK_NULL_HANDLE };
     VkDeviceAddress    address{ 0 };
     VkDeviceAddress    opaque_address{ 0 };
     VkBufferUsageFlags usage{ 0 };
 
     std::set<BufferViewWrapper*> buffer_views;
+
+    bool                                       is_sparse_buffer{ false };
+    std::map<VkDeviceSize, VkSparseMemoryBind> sparse_memory_bind_map;
+    VkQueue                                    sparse_bind_queue;
+
+    VkDeviceSize created_size{ 0 };
+
+    std::unordered_map<VkDeviceAddress, AccelerationStructureBuildState> acceleration_structures;
+
+    // optional opaque descriptor-data used by VK_EXT_descriptor_buffer
+    std::vector<uint8_t> opaque_descriptor_data;
 };
 
 struct ImageViewWrapper;
@@ -231,11 +249,25 @@ struct ImageWrapper : public HandleWrapper<VkImage>, AssetWrapperBase
     bool                  is_swapchain_image{ false };
 
     std::set<ImageViewWrapper*> image_views;
+
+    bool                                                is_sparse_image{ false };
+    std::map<VkDeviceSize, VkSparseMemoryBind>          sparse_opaque_memory_bind_map;
+    graphics::VulkanSubresourceSparseImageMemoryBindMap sparse_subresource_memory_bind_map;
+    VkQueue                                             sparse_bind_queue;
+
+    std::set<VkSwapchainKHR> parent_swapchains;
+
+    // optional opaque descriptor-data used by VK_EXT_descriptor_buffer
+    std::vector<uint8_t> opaque_descriptor_data;
 };
 
 struct SamplerWrapper : public HandleWrapper<VkSampler>
 {
+    format::HandleId                          device_id{ format::kNullHandleId };
     std::unordered_set<DescriptorSetWrapper*> descriptor_sets_bound_to;
+
+    // optional opaque descriptor-data used by VK_EXT_descriptor_buffer
+    std::vector<uint8_t> opaque_descriptor_data;
 };
 
 struct DeviceMemoryWrapper : public HandleWrapper<VkDeviceMemory>
@@ -245,7 +277,7 @@ struct DeviceMemoryWrapper : public HandleWrapper<VkDeviceMemory>
     // This is the device which was used to allocate the memory.
     // Spec states if the memory can be mapped, the mapping device must be this device.
     // The device wrapper will be initialized when allocating the memory. Some handling
-    // like StateTracker::TrackTlasToBlasDependencies may use it before mapping
+    // like StateTracker::TrackCommandBuffersSubmision may use it before mapping
     // the memory.
     DeviceWrapper*   parent_device{ nullptr };
     const void*      mapped_data{ nullptr };
@@ -264,6 +296,10 @@ struct DeviceMemoryWrapper : public HandleWrapper<VkDeviceMemory>
 
     std::unordered_set<AssetWrapperBase*> bound_assets;
     std::mutex                            asset_map_lock;
+
+    // optional deep-copy of modified allocate-info, required for state-tracking / trimming
+    VkMemoryAllocateInfo*      modified_allocation_info{ nullptr };
+    std::unique_ptr<uint8_t[]> modified_allocation_info_data{ nullptr };
 };
 
 struct BufferViewWrapper : public HandleWrapper<VkBufferView>
@@ -276,10 +312,14 @@ struct BufferViewWrapper : public HandleWrapper<VkBufferView>
 
 struct ImageViewWrapper : public HandleWrapper<VkImageView>
 {
+    format::HandleId device_id{ format::kNullHandleId };
     format::HandleId image_id{ format::kNullHandleId };
     ImageWrapper*    image{ nullptr };
 
     std::unordered_set<DescriptorSetWrapper*> descriptor_sets_bound_to;
+
+    // optional opaque descriptor-data used by VK_EXT_descriptor_buffer
+    std::vector<uint8_t> opaque_descriptor_data;
 };
 
 struct FramebufferWrapper : public HandleWrapper<VkFramebuffer>
@@ -407,6 +447,20 @@ struct CommandBufferWrapper : public HandleWrapper<VkCommandBuffer>
     util::MemoryOutputStream   command_data;
     std::set<format::HandleId> command_handles[vulkan_state_info::CommandHandleType::NumHandleTypes];
 
+    enum CommandBufferState
+    {
+        kInitial = 0,
+        kRecording,
+        kExecutable,
+        kInvalid
+    };
+
+    CommandBufferState state{ kInitial };
+
+    // This is set to true when a command buffer is began with VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT
+    // Used to track command buffer life cycle
+    bool one_time_submission{ false };
+
     // Image layout info tracked for image barriers recorded to the command buffer. To be updated on calls to
     // vkCmdPipelineBarrier and vkCmdEndRenderPass and applied to the image wrapper on calls to vkQueueSubmit. To be
     // transferred from secondary command buffers to primary command buffers on calls to vkCmdExecuteCommands.
@@ -477,6 +531,13 @@ struct CommandPoolWrapper : public HandleWrapper<VkCommandPool>
 
 // For vkGetPhysicalDeviceSurfaceCapabilitiesKHR
 struct SurfaceCapabilities
+{
+    VkSurfaceKHR             surface;
+    VkSurfaceCapabilitiesKHR surface_capabilities;
+};
+
+// For vkGetPhysicalDeviceSurfaceCapabilities2KHR
+struct SurfaceCapabilities2
 {
     VkPhysicalDeviceSurfaceInfo2KHR surface_info;
     std::unique_ptr<uint8_t[]>      surface_info_pnext_memory;
@@ -487,6 +548,13 @@ struct SurfaceCapabilities
 
 // For vkGetPhysicalDeviceSurfaceFormatsKHR
 struct SurfaceFormats
+{
+    VkSurfaceKHR                    surface{ VK_NULL_HANDLE };
+    std::vector<VkSurfaceFormatKHR> surface_formats;
+};
+
+// For vkGetPhysicalDeviceSurfaceFormats2KHR
+struct SurfaceFormats2
 {
     VkPhysicalDeviceSurfaceInfo2KHR surface_info;
     std::unique_ptr<uint8_t[]>      surface_info_pnext_memory;
@@ -517,10 +585,10 @@ struct SurfaceKHRWrapper : public HandleWrapper<VkSurfaceKHR>
     // Track results from calls to vkGetPhysicalDeviceSurfaceSupportKHR to write to the state snapshot after surface
     // creation. The call is only written to the state snapshot if it was previously called by the application.
     // Keys are the VkPhysicalDevice handle ID.
-    std::unordered_map<format::HandleId, std::unordered_map<uint32_t, VkBool32>> surface_support;
-    std::unordered_map<format::HandleId, SurfaceCapabilities>                    surface_capabilities;
-    std::unordered_map<format::HandleId, SurfaceFormats>                         surface_formats;
-    std::unordered_map<format::HandleId, SurfacePresentModes>                    surface_present_modes;
+    std::unordered_map<format::HandleId, std::unordered_map<uint32_t, VkBool32>>                  surface_support;
+    std::unordered_map<format::HandleId, std::variant<SurfaceCapabilities, SurfaceCapabilities2>> surface_capabilities;
+    std::unordered_map<format::HandleId, std::variant<SurfaceFormats, SurfaceFormats2>>           surface_formats;
+    std::unordered_map<format::HandleId, SurfacePresentModes>                                     surface_present_modes;
 
     // Keys are the VkDevice handle ID.
     std::unordered_map<format::HandleId, GroupSurfacePresentModes> group_surface_present_modes;
@@ -562,55 +630,17 @@ struct AccelerationStructureKHRWrapper : public HandleWrapper<VkAccelerationStru
     std::vector<AccelerationStructureKHRWrapper*> blas;
 
     VkAccelerationStructureTypeKHR type;
+
+    // associated buffer
+    BufferWrapper* buffer = nullptr;
+    VkDeviceSize   offset = 0;
+    VkDeviceSize   size   = 0;
+
     // Only used when tracking
-
-    struct ASInputBuffer
-    {
-        // Required data to correctly create a buffer
-        VkBuffer           handle{ VK_NULL_HANDLE };
-        format::HandleId   handle_id{ format::kNullHandleId };
-        DeviceWrapper*     bind_device{ nullptr };
-        uint32_t           queue_family_index{ 0 };
-        VkDeviceSize       created_size{ 0 };
-        VkBufferUsageFlags usage{ 0 };
-
-        bool destroyed{ false };
-
-        VkDeviceAddress capture_address{ 0 };
-        VkDeviceAddress actual_address{ 0 };
-
-        std::vector<uint8_t> bytes;
-
-        VkMemoryRequirements memory_requirements{};
-        format::HandleId     bind_memory{};
-        VkDeviceMemory       bind_memory_handle{ VK_NULL_HANDLE };
-    };
-
-    struct AccelerationStructureKHRBuildCommandData
-    {
-        VkAccelerationStructureBuildGeometryInfoKHR           geometry_info;
-        std::unique_ptr<uint8_t[]>                            geometry_info_memory;
-        std::vector<VkAccelerationStructureBuildRangeInfoKHR> build_range_infos;
-        std::unordered_map<format::HandleId, ASInputBuffer>   input_buffers;
-    };
-    std::optional<AccelerationStructureKHRBuildCommandData> latest_update_command_{ std::nullopt };
-    std::optional<AccelerationStructureKHRBuildCommandData> latest_build_command_{ std::nullopt };
-
-    struct AccelerationStructureCopyCommandData
-    {
-        format::HandleId                   device;
-        VkCopyAccelerationStructureInfoKHR info;
-    };
-    std::optional<AccelerationStructureCopyCommandData> latest_copy_command_{ std::nullopt };
-
-    struct AccelerationStructureWritePropertiesCommandData
-    {
-        format::HandleId device;
-        VkQueryType      query_type;
-    };
-    std::optional<AccelerationStructureWritePropertiesCommandData> latest_write_properties_command_{ std::nullopt };
-
     std::unordered_set<DescriptorSetWrapper*> descriptor_sets_bound_to;
+
+    // optional opaque descriptor-data used by VK_EXT_descriptor_buffer
+    std::vector<uint8_t> opaque_descriptor_data;
 };
 
 struct AccelerationStructureNVWrapper : public HandleWrapper<VkAccelerationStructureNV>
@@ -635,6 +665,12 @@ struct PipelineCacheWrapper : public HandleWrapper<VkPipelineCache>
     std::vector<uint8_t>      cache_data;
 };
 
+struct DataGraphPipelineSessionARMWrapper : public HandleWrapper<VkDataGraphPipelineSessionARM>, AssetWrapperBase
+{
+    VkDataGraphPipelineSessionBindPointARM bind_point;
+    uint32_t                               object_index;
+};
+
 // Handle alias types for extension handle types that have been promoted to core types.
 typedef SamplerYcbcrConversionWrapper   SamplerYcbcrConversionKHRWrapper;
 typedef DescriptorUpdateTemplateWrapper DescriptorUpdateTemplateKHRWrapper;
