@@ -55,18 +55,19 @@ class VulkanStateTracker
 
     ~VulkanStateTracker();
 
-    uint64_t WriteState(util::FileOutputStream*           file_stream,
-                        util::ThreadData*                 thread_data,
-                        std::function<format::HandleId()> get_unique_id_fn,
-                        util::Compressor*                 compressor,
-                        uint64_t                          frame_number,
-                        util::FileOutputStream*           asset_file_stream,
-                        const std::string*                asset_file_name)
+    uint64_t WriteState(util::FileOutputStream*          file_stream,
+                        util::ThreadData*                thread_data,
+                        vulkan_wrappers::PFN_GetHandleId get_unique_id_fn,
+                        util::Compressor*                compressor,
+                        uint64_t                         frame_number,
+                        util::FileOutputStream*          asset_file_stream,
+                        const std::string*               asset_file_name)
     {
         VulkanStateWriter state_writer(file_stream,
                                        compressor,
                                        thread_data,
                                        get_unique_id_fn,
+                                       device_address_trackers_,
                                        asset_file_stream,
                                        asset_file_name,
                                        asset_file_stream != nullptr ? &asset_file_offsets_ : nullptr);
@@ -75,11 +76,11 @@ class VulkanStateTracker
         return state_writer.WriteState(state_table_, frame_number);
     }
 
-    uint64_t WriteAssets(util::FileOutputStream*           asset_file_stream,
-                         const std::string*                asset_file_name,
-                         util::ThreadData*                 thread_data,
-                         std::function<format::HandleId()> get_unique_id_fn,
-                         util::Compressor*                 compressor)
+    uint64_t WriteAssets(util::FileOutputStream*          asset_file_stream,
+                         const std::string*               asset_file_name,
+                         util::ThreadData*                thread_data,
+                         vulkan_wrappers::PFN_GetHandleId get_unique_id_fn,
+                         util::Compressor*                compressor)
     {
         assert(asset_file_stream != nullptr);
         assert(asset_file_name != nullptr);
@@ -88,6 +89,7 @@ class VulkanStateTracker
                                        compressor,
                                        thread_data,
                                        get_unique_id_fn,
+                                       device_address_trackers_,
                                        asset_file_stream,
                                        asset_file_name,
                                        &asset_file_offsets_);
@@ -385,8 +387,7 @@ class VulkanStateTracker
                            void*            mapped_data,
                            VkDeviceSize     mapped_offset,
                            VkDeviceSize     mapped_size,
-                           VkMemoryMapFlags mapped_flags,
-                           bool             track_assets);
+                           VkMemoryMapFlags mapped_flags);
 
     void TrackBeginRenderPass(VkCommandBuffer command_buffer, const VkRenderPassBeginInfo* begin_info);
 
@@ -450,6 +451,9 @@ class VulkanStateTracker
                               const uint32_t*       image_indices,
                               VkQueue               queue);
 
+    // finds potential fences in pNext-chain and tracks their usage
+    void TrackPresentFences(const VkPresentInfoKHR* present_info);
+
     void TrackAccelerationStructureKHRDeviceAddress(VkDevice                   device,
                                                     VkAccelerationStructureKHR accel_struct,
                                                     VkDeviceAddress            address);
@@ -479,6 +483,9 @@ class VulkanStateTracker
         VkPhysicalDevice                                    physicalDevice,
         VkPhysicalDeviceAccelerationStructurePropertiesKHR* acceleration_structure_properties);
 
+    void TrackDescriptorBufferProperties(VkPhysicalDevice                               physicalDevice,
+                                         VkPhysicalDeviceDescriptorBufferPropertiesEXT* descriptor_buffer_properties);
+
     void TrackRayTracingShaderGroupHandles(VkDevice device, VkPipeline pipeline, size_t data_size, const void* data);
 
     void TrackAcquireFullScreenExclusiveMode(VkDevice device, VkSwapchainKHR swapchain);
@@ -493,7 +500,7 @@ class VulkanStateTracker
 
     void TrackSetLocalDimmingAMD(VkDevice device, VkSwapchainKHR swapChain, VkBool32 localDimmingEnable);
 
-    void TrackTlasToBlasDependencies(uint32_t command_buffer_count, const VkCommandBuffer* command_buffers);
+    void TrackCommandBuffersSubmision(uint32_t command_buffer_count, const VkCommandBuffer* command_buffers);
 
     void TrackCmdBindDescriptorSets(VkCommandBuffer        commandBuffer,
                                     VkPipelineBindPoint    pipelineBindPoint,
@@ -743,6 +750,8 @@ class VulkanStateTracker
 
     void TrackAssetsInSubmission(uint32_t submitCount, const VkSubmitInfo2* pSubmits);
 
+    void TrackAssetsInMemory(format::HandleId memory_id);
+
     void TrackBeginRendering(VkCommandBuffer commandBuffer, const VkRenderingInfo* pRenderingInfo);
 
     void TrackSetDebugUtilsObjectNameEXT(VkDevice                             device,
@@ -753,6 +762,8 @@ class VulkanStateTracker
                                         const VkDebugUtilsObjectTagInfoEXT* pTagInfo,
                                         const util::MemoryOutputStream*     object_tag_parameter_buffer);
 
+    void TrackBeginCommandBuffer(VkCommandBuffer command_buffer, VkCommandBufferUsageFlags flags);
+
   private:
     template <typename ParentHandle, typename SecondaryHandle, typename Wrapper, typename CreateInfo>
     void AddGroupHandles(ParentHandle                        parent_handle,
@@ -816,7 +827,7 @@ class VulkanStateTracker
 
     void DestroyState(vulkan_wrappers::DeviceMemoryWrapper* wrapper);
 
-    void DestroyState(vulkan_wrappers::BufferWrapper* wrapper);
+    void DestroyState(vulkan_wrappers::BufferWrapper* buffer_wrapper);
 
     void DestroyState(vulkan_wrappers::AccelerationStructureKHRWrapper* wrapper);
 
