@@ -161,6 +161,11 @@ struct LARGE_INTEGER
 
 #endif // WIN32
 
+static inline int64_t pack_luid(LUID luid)
+{
+    return static_cast<int64_t>((static_cast<uint64_t>(luid.HighPart) << 32) | luid.LowPart);
+}
+
 #ifndef __ANDROID__
 typedef void* jobject;
 #endif
