@@ -31,6 +31,8 @@
 #include "vulkan/vulkan_core.h"
 
 #include <vector>
+#include <functional>
+#include <map>
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(graphics)
@@ -59,8 +61,8 @@ class VulkanResourcesUtil
     // resource and the size of the biggest staging buffer necessary is known in advance.
     VkResult CreateStagingBuffer(VkDeviceSize size);
 
-    // Will return the size requirements and offsets for each subresource contained in the specified image.
-    // Sizes and offsets are calculated in such a way that the each subresource will be tightly packed.
+    // Will return the size requirements and offsets for each subresource contained for an image with the specified
+    // attributes. Sizes and offsets are calculated in such a way that the each subresource will be tightly packed.
     //
     // The sizes are returned in the subresource_sizes vector and will be in the order:
     //    M0 L0 L1 ... La M1 L0 L1 ... La ... Mm L0 L1 ... La
@@ -69,8 +71,7 @@ class VulkanResourcesUtil
     // all_layers_per_level boolean determines if all array layer per mip map level will be accounted as one.
     //
     // Return value is the total size of the image.
-    uint64_t GetImageResourceSizesOptimal(VkImage                image,
-                                          VkFormat               format,
+    uint64_t GetImageResourceSizesOptimal(VkFormat               format,
                                           VkImageType            type,
                                           const VkExtent3D&      extent,
                                           uint32_t               mip_levels,
@@ -228,6 +229,7 @@ class VulkanResourcesUtil
                           VkImage           image,
                           VkFormat          format,
                           VkImageType       type,
+                          VkImageTiling     tiling,
                           const VkExtent3D& extent,
                           uint32_t          array_layers,
                           VkImageLayout     current_layout,
@@ -288,7 +290,9 @@ void GetFormatAspects(VkFormat                            format,
                       std::vector<VkImageAspectFlagBits>* aspects,
                       bool*                               combined_depth_stencil = nullptr);
 
-VkImageAspectFlags GetFormatAspectMask(VkFormat format);
+void AspectFlagsToFlagBits(VkImageAspectFlags aspect_mask, std::vector<VkImageAspectFlagBits>& aspects);
+
+VkImageAspectFlags GetFormatAspects(VkFormat format);
 
 VkFormat GetImageAspectFormat(VkFormat format, VkImageAspectFlagBits aspect);
 
@@ -298,6 +302,84 @@ bool FindMemoryTypeIndex(const VkPhysicalDeviceMemoryProperties& memory_properti
                          uint32_t*                               found_index,
                          VkMemoryPropertyFlags*                  found_flags);
 
+struct VkOffset3DComparator
+{
+    bool operator()(const VkOffset3D& l, const VkOffset3D& r) const
+    {
+        bool result = (l.x < r.x);
+
+        if (l.x == r.x)
+        {
+            result = (l.y < r.y);
+
+            if (l.y == r.y)
+            {
+                result = (l.z < r.z);
+            }
+        }
+
+        return result;
+    }
+};
+
+typedef std::map<VkOffset3D, VkSparseImageMemoryBind, gfxrecon::graphics::VkOffset3DComparator>
+    VulkanOffset3DSparseImageMemoryBindMap;
+
+struct VkImageSubresourceComparator
+{
+    bool operator()(const VkImageSubresource& l, const VkImageSubresource& r) const
+    {
+        bool result = (l.arrayLayer < r.arrayLayer);
+
+        if (l.arrayLayer == r.arrayLayer)
+        {
+            result = (l.mipLevel < r.mipLevel);
+
+            if (l.mipLevel == r.mipLevel)
+            {
+                result = (l.aspectMask < r.aspectMask);
+            }
+        }
+
+        return result;
+    }
+};
+
+typedef std::
+    map<VkImageSubresource, VulkanOffset3DSparseImageMemoryBindMap, gfxrecon::graphics::VkImageSubresourceComparator>
+        VulkanSubresourceSparseImageMemoryBindMap;
+
+// Get the intersection of the new bind range and the existing bind range for sparse buffer or sparse image (opaque
+// bind).
+// If the intersection range exists, further get the remaining ranges for the existing bind range after removing
+// the intersection range. For instance, if the new/existing bind range (offset, size)  are (196608, 327680) and (0,
+// 655360) respectively, the old range (0, 655360) completely covers the new range (196608, 327680). The intersection
+// range is (196608, 327680), and the remaining ranges for the existing bind are (0, 196608) and (524288, 131072). So
+// for the return of the function, remaining_resource_offsets will be a std::vector of [0, 524288], and
+// remaining_resource_sizes will be [196608, 131072].
+//
+bool GetIntersectForSparseMemoryBind(uint32_t               new_bind_resource_offset,
+                                     uint32_t               new_bind_resource_size,
+                                     uint32_t               existing_bind_resource_offset,
+                                     uint32_t               existing_bind_resource_size,
+                                     uint32_t&              intersection_resource_offset,
+                                     uint32_t&              intersection_resource_size,
+                                     std::vector<uint32_t>& remaining_resource_offsets,
+                                     std::vector<uint32_t>& remaining_resource_sizes,
+                                     bool&                  new_bind_range_include_existing_bind_tange,
+                                     bool&                  existing_bind_range_include_new_bind_tange);
+
+void UpdateSparseMemoryBindMap(std::map<VkDeviceSize, VkSparseMemoryBind>& sparse_memory_bind_map,
+                               const VkSparseMemoryBind&                   new_sparse_memory_bind);
+
+void GetIntersectForSparseImageMemoryBind(const VkSparseImageMemoryBind&        old_bind,
+                                          const VkSparseImageMemoryBind&        new_bind,
+                                          std::vector<VkSparseImageMemoryBind>& new_binds,
+                                          std::vector<VkOffset3D>&              removed_binds);
+
+void UpdateSparseImageMemoryBindMap(VulkanSubresourceSparseImageMemoryBindMap& sparse_image_memory_bind_map,
+                                    const VkSparseImageMemoryBind&             new_bind);
+
 bool GetImageTexelSize(VkFormat      format,
                        VkDeviceSize* texel_size,
                        bool*         is_texel_block_size,
@@ -341,7 +423,19 @@ bool NextRowTexelCoordinates(VkImageType       imageType,
                              uint32_t&         z,
                              uint32_t&         layer);
 
+/**
+ * @brief Get the size requirements for a staging buffer to copy image data from a buffer
+ * @see GetBufferSizeFromCopyImage(RegionCopy&, uint32_t, VkFormat) in `vulkan_resources_util.cpp`
+ */
+VkDeviceSize GetBufferSizeFromCopyImage(const VkMemoryToImageCopy& region, uint32_t array_layers, VkFormat format);
+
+/**
+ * @brief Get the size requirements for a staging buffer to copy image data to a buffer
+ * @see GetBufferSizeFromCopyImage(RegionCopy&, uint32_t, VkFormat) in `vulkan_resources_util.cpp`
+ */
+VkDeviceSize GetBufferSizeFromCopyImage(const VkImageToMemoryCopy& region, uint32_t array_layers, VkFormat format);
+
+GFXRECON_END_NAMESPACE(graphics)
 GFXRECON_END_NAMESPACE(gfxrecon)
-GFXRECON_END_NAMESPACE(encode)
 
 #endif /* GFXRECON_GRAPHICS_VULKAN_RESOURCES_UTIL_H */
