@@ -20,65 +20,16 @@
 ** DEALINGS IN THE SOFTWARE.
 */
 
+#include "wait_for_present_app.h"
+
 #include <iostream>
 
 #include <vulkan/vulkan_core.h>
 
-#include <test_app_base.h>
-
-#include <SDL3/SDL_main.h>
-
-namespace gfxrecon
-{
-
-namespace test_app
-{
-
+GFXRECON_BEGIN_NAMESPACE(gfxrecon)
+GFXRECON_BEGIN_NAMESPACE(test_app)
 // WaitForPresent requires Linux Mesa driver.
-namespace wait_for_present
-{
-
-const size_t MAX_FRAMES_IN_FLIGHT = 2;
-
-class App : public gfxrecon::test::TestAppBase
-{
-  public:
-    App() = default;
-
-  private:
-    VkPhysicalDevicePresentIdFeaturesKHR   present_id_features_;
-    VkPhysicalDevicePresentWaitFeaturesKHR present_wait_features_;
-
-    VkQueue graphics_queue_;
-    VkQueue present_queue_;
-
-    std::vector<VkFramebuffer> framebuffers_;
-
-    VkRenderPass     render_pass_;
-    VkPipelineLayout pipeline_layout_;
-    VkPipeline       graphics_pipeline_;
-
-    VkCommandPool command_pools_[MAX_FRAMES_IN_FLIGHT];
-
-    size_t current_frame_ = 0;
-
-    gfxrecon::test::Sync sync_;
-
-    void create_render_pass();
-    void create_graphics_pipeline();
-    void create_framebuffers();
-    void recreate_swapchain();
-    void cleanup() override;
-    bool frame(const int frame_num) override;
-    void setup() override;
-
-    void configure_instance_builder(test::InstanceBuilder& instance_builder, vkmock::TestConfig*) override;
-    void configure_physical_device_selector(test::PhysicalDeviceSelector& phys_device_selector,
-                                            vkmock::TestConfig*) override;
-    void configure_device_builder(test::DeviceBuilder&        device_builder,
-                                  test::PhysicalDevice const& physical_device,
-                                  vkmock::TestConfig*         test_config) override;
-};
+GFXRECON_BEGIN_NAMESPACE(wait_for_present)
 
 void App::configure_instance_builder(test::InstanceBuilder& instance_builder, vkmock::TestConfig* test_config)
 {
@@ -148,8 +99,15 @@ void App::create_render_pass()
 
 void App::create_graphics_pipeline()
 {
-    auto vert_module = gfxrecon::test::readShaderFromFile(init.disp, "shaders/vert.spv");
-    auto frag_module = gfxrecon::test::readShaderFromFile(init.disp, "shaders/frag.spv");
+#ifdef __ANDROID__
+    auto vert_module =
+        gfxrecon::test::readShaderFromFile(init.disp, "wait-for-present/shaders/vert.spv", init.android_app);
+    auto frag_module =
+        gfxrecon::test::readShaderFromFile(init.disp, "wait-for-present/shaders/frag.spv", init.android_app);
+#else
+    auto vert_module = gfxrecon::test::readShaderFromFile(init.disp, "wait-for-present/shaders/vert.spv");
+    auto frag_module = gfxrecon::test::readShaderFromFile(init.disp, "wait-for-present/shaders/frag.spv");
+#endif
 
     VkPipelineShaderStageCreateInfo vert_stage_info = {};
     vert_stage_info.sType                           = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO;
@@ -532,23 +490,6 @@ void App::setup()
     sync_ = gfxrecon::test::create_sync_objects(init.swapchain, init.disp, MAX_FRAMES_IN_FLIGHT);
 }
 
-} // namespace wait_for_present
-
-} // namespace test_app
-
-} // namespace gfxrecon
-
-int main(int argc, char* argv[])
-{
-    try
-    {
-        gfxrecon::test_app::wait_for_present::App app{};
-        app.run("wait-for-present");
-        return 0;
-    }
-    catch (const std::exception& e)
-    {
-        std::cout << e.what() << std::endl;
-        return -1;
-    }
-}
+GFXRECON_END_NAMESPACE(wait_for_present)
+GFXRECON_END_NAMESPACE(test_app)
+GFXRECON_END_NAMESPACE(gfxrecon)
