@@ -93,7 +93,7 @@ bool FileProcessorSucceeded(const decode::FileProcessor& processor)
         GFXRECON_WRITE_CONSOLE("Did not detect any frames in the capture.");
     }
 
-    if ((processor.GetErrorState() == gfxrecon::decode::FileProcessor::kErrorNone) == false)
+    if ((processor.GetErrorState() == gfxrecon::decode::BlockIOError::kErrorNone) == false)
     {
         GFXRECON_WRITE_CONSOLE("Encountered error while reading the capture.");
     }
@@ -104,7 +104,7 @@ bool FileProcessorSucceeded(const decode::FileProcessor& processor)
     }
 
     return (processor.GetCurrentFrameNumber() > 0) &&
-           (processor.GetErrorState() == gfxrecon::decode::FileProcessor::kErrorNone) &&
+           (processor.GetErrorState() == gfxrecon::decode::BlockIOError::kErrorNone) &&
            processor.EntireFileWasProcessed();
 }
 
@@ -160,7 +160,7 @@ bool GetPsoOptimizationInfo(const std::string&               input_filename,
                 options.optimize_resource_values = false;
             }
         }
-        else if (pso_pass_file_processor.GetErrorState() != gfxrecon::decode::FileProcessor::kErrorNone)
+        else if (pso_pass_file_processor.GetErrorState() != gfxrecon::decode::BlockIOError::kErrorNone)
         {
             GFXRECON_WRITE_CONSOLE("A failure has occurred during scanning capture file for unreferenced PSOs");
         }
@@ -262,7 +262,7 @@ bool GetDxrOptimizationInfo(const std::string&               input_filename,
 
             dxr_scan_result = true;
         }
-        else if (dxr_pass_file_processor.GetErrorState() != gfxrecon::decode::FileProcessor::kErrorNone)
+        else if (dxr_pass_file_processor.GetErrorState() != gfxrecon::decode::BlockIOError::kErrorNone)
         {
             GFXRECON_WRITE_CONSOLE("A failure has occurred during capture processing for DXR/EI optimization");
         }
@@ -397,7 +397,7 @@ bool ApplyDx12OptimizationInfo(const std::string&                     input_file
 
             file_optimizer.Process();
 
-            if (file_optimizer.GetErrorState() != gfxrecon::FileOptimizer::kErrorNone)
+            if (file_optimizer.GetErrorState() != gfxrecon::decode::BlockIOError::kErrorNone)
             {
                 GFXRECON_WRITE_CONSOLE("A failure has occurred during capture processing (error=%d). If it was "
                                        "created, the output file may be invalid.",
