@@ -136,7 +136,7 @@ static bool CheckOptionPrintVersion(const char* exe_name, const gfxrecon::util::
         }
 
         GFXRECON_WRITE_CONSOLE("%s version info:", app_name.c_str());
-        GFXRECON_WRITE_CONSOLE("  GFXReconstruct Version %s", GFXRECON_PROJECT_VERSION_STRING);
+        GFXRECON_WRITE_CONSOLE("  GFXReconstruct Version %s", GetProjectVersionString());
         GFXRECON_WRITE_CONSOLE("  Vulkan Header Version %u.%u.%u",
                                VK_VERSION_MAJOR(VK_HEADER_VERSION_COMPLETE),
                                VK_VERSION_MINOR(VK_HEADER_VERSION_COMPLETE),
@@ -370,7 +370,7 @@ bool ProcessCapture(gfxrecon::decode::VulkanCppConsumer&      cpp_consumer,
     } while (success && file_processor.GetCurrentFrameNumber() <= frame_limit);
     printf("\nDone processing file\n");
 
-    return (file_processor.GetErrorState() == file_processor.kErrorNone);
+    return (file_processor.GetErrorState() == gfxrecon::decode::BlockIOError::kErrorNone);
 }
 
 int main(int argc, const char** argv)
